

import os
import shutil
from six.moves.urllib.request import urlretrieve

TUTORIAL_DIR = 'AltairTutorial'

BASE_URL = 'https://raw.githubusercontent.com/ellisonbg/altair/master/notebooks/'

FILES = [
    '01-Index.ipynb',
    '02-Introduction.ipynb',
    '03-ScatterCharts.ipynb',
    '04-BarCharts.ipynb',
    '05-LineCharts.ipynb',
    '06-AreaCharts.ipynb',
    '07-LayeredCharts.ipynb',
    '08-GroupedRegressionCharts.ipynb',
    '09-CarsDataset.ipynb',
    '10-IrisPairgrid.ipynb'
]
 
 
def download_tutorial(overwrite=True):
    """Download the Altair tutorial notebooks."""
    if os.path.isdir(TUTORIAL_DIR) and overwrite:
        print('Removing old tutorial directory: {}'.format(TUTORIAL_DIR))
        shutil.rmtree(TUTORIAL_DIR, ignore_errors=True)
    print('Downloading fresh tutorial directory: {}'.format(TUTORIAL_DIR))
    os.mkdir(TUTORIAL_DIR)
    for f in FILES:
        urlretrieve(BASE_URL+f, os.path.join(TUTORIAL_DIR, f))


def tutorial(overwrite=True):
    """Download the Altair tutorial notebooks and show a link in the notebook."""
    download_tutorial(overwrite=overwrite)
    print('Click on the following notebooks to explore the tutorial:')
    from IPython.display import FileLinks, display
    display(FileLinks(TUTORIAL_DIR))



import pytest
import warnings
import json

import numpy as np
import pandas as pd

from .. import (parse_shorthand, construct_shorthand,
                infer_vegalite_type, sanitize_dataframe)


def test_parse_shorthand():
    def check(s, **kwargs):
        assert parse_shorthand(s) == kwargs

    check('')

    # Fields alone
    check('foobar', field='foobar')
    check('blah:(fd ', field='blah:(fd ')

    # Fields with type
    check('foobar:quantitative', type='quantitative', field='foobar')
    check('foobar:nominal', type='nominal', field='foobar')
    check('foobar:ordinal', type='ordinal', field='foobar')
    check('foobar:temporal', type='temporal', field='foobar')

    check('foobar:Q', type='quantitative', field='foobar')
    check('foobar:N', type='nominal', field='foobar')
    check('foobar:O', type='ordinal', field='foobar')
    check('foobar:T', type='temporal', field='foobar')

    # Fields with aggregate and/or type
    check('average(foobar)', field='foobar', aggregate='average')
    check('min(foobar):temporal', type='temporal', field='foobar', aggregate='min')
    check('sum(foobar):Q', type='quantitative', field='foobar', aggregate='sum')

    # check that invalid arguments are not split-out
    check('invalid(blah)', field='invalid(blah)')
    check('blah:invalid', field='blah:invalid')
    check('invalid(blah):invalid', field='invalid(blah):invalid')

    # check parsing in presence of strange characters
    check('average(a b:(c\nd):Q', aggregate='average',
          field='a b:(c\nd', type='quantitative')


def test_shorthand_roundtrip():
    def check(**kwargs):
        assert parse_shorthand(construct_shorthand(**kwargs)) == kwargs

    check(field='foo')
    check(field='foo', aggregate='average')
    check(field='foo', type='quantitative')
    check(field='foo', aggregate='average', type='quantitative')


def test_infer_vegalite_type():
    def _check(arr, typ):
        assert infer_vegalite_type(arr) == typ

    _check(np.arange(5, dtype=float), 'quantitative')
    _check(np.arange(5, dtype=int), 'quantitative')
    _check(np.zeros(5, dtype=bool), 'nominal')
    _check(pd.date_range('2012', '2013'), 'temporal')
    _check(pd.timedelta_range(365, periods=12), 'temporal')

    nulled = pd.Series(np.random.randint(10, size=10))
    nulled[0] = None
    _check(nulled, 'quantitative')
    _check(['a', 'b', 'c'], 'nominal')

    if hasattr(pytest, 'warns'):  # added in pytest 2.8
        with pytest.warns(UserWarning):
            _check([], 'nominal')
    else:
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore")
            _check([], 'nominal')


def test_sanitize_dataframe():
    # create a dataframe with various types
    df = pd.DataFrame({'s': list('abcde'),
                       'f': np.arange(5, dtype=float),
                       'i': np.arange(5, dtype=int),
                       'd': pd.date_range('2012-01-01', periods=5, freq='H'),
                       'c': pd.Series(list('ababc'), dtype='category')})

    # add some nulls
    df.ix[0, 's'] = None
    df.ix[0, 'f'] = np.nan
    df.ix[0, 'd'] = pd.NaT

    # JSON serialize. This will fail on non-sanitized dataframes
    df_clean = sanitize_dataframe(df)
    s = json.dumps(df_clean.to_dict(orient='records'))

    # Re-construct pandas dataframe
    df2 = pd.read_json(s)

    # Re-apply original types
    for col in df:
        if str(df[col].dtype).startswith('datetime'):
            # astype(datetime) introduces time-zone issues:
            # to_datetime() does not.
            df2[col] = pd.to_datetime(df2[col])
        else:
            df2[col] = df2[col].astype(df[col].dtype)

    assert df.equals(df2)



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject


class Bin(BaseObject):
    """Wrapper for Vega-Lite Bin definition.
    
    Attributes
    ----------
    base: CFloat
        The number base to use for automatic bin determination (default is base 10).
    div: List(CFloat)
        Scale factors indicating allowable subdivisions.
    max: CFloat
        The maximum bin value to consider.
    maxbins: CFloat
        Maximum number of bins.
    min: CFloat
        The minimum bin value to consider.
    minstep: CFloat
        A minimum allowable step size (particularly useful for integer values).
    step: CFloat
        An exact step size to use between bins.
    steps: List(CFloat)
        An array of allowable step sizes to choose from.
    """
    base = T.CFloat(allow_none=True, default_value=None, help="""The number base to use for automatic bin determination (default is base 10).""")
    div = T.List(T.CFloat(), allow_none=True, default_value=None, help="""Scale factors indicating allowable subdivisions.""")
    max = T.CFloat(allow_none=True, default_value=None, help="""The maximum bin value to consider.""")
    maxbins = T.CFloat(allow_none=True, default_value=None, min=2, help="""Maximum number of bins.""")
    min = T.CFloat(allow_none=True, default_value=None, help="""The minimum bin value to consider.""")
    minstep = T.CFloat(allow_none=True, default_value=None, help="""A minimum allowable step size (particularly useful for integer values).""")
    step = T.CFloat(allow_none=True, default_value=None, help="""An exact step size to use between bins.""")
    steps = T.List(T.CFloat(), allow_none=True, default_value=None, help="""An array of allowable step sizes to choose from.""")
    
    def __init__(self, base=None, div=None, max=None, maxbins=None, min=None, minstep=None, step=None, steps=None, **kwargs):
        kwds = dict(base=base, div=div, max=max, maxbins=maxbins, min=min, minstep=minstep, step=step, steps=steps)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Bin, self).__init__(**kwargs)


"""Wrappers for low-level schema objects"""


__all__ = [
             "PositionChannel",
             "ChannelWithLegend",
             "Field",
             "OrderChannel",
             "Color",
             "Column",
             "Detail",
             "Label",
             "Opacity",
             "Order",
             "Path",
             "Row",
             "Shape",
             "Size",
             "Text",
             "X",
             "Y",
             "Encoding",
             "Facet",
          ]


from .channel_wrappers import PositionChannel
from .channel_wrappers import ChannelWithLegend
from .channel_wrappers import Field
from .channel_wrappers import OrderChannel
from .named_channels import Color
from .named_channels import Column
from .named_channels import Detail
from .named_channels import Label
from .named_channels import Opacity
from .named_channels import Order
from .named_channels import Path
from .named_channels import Row
from .named_channels import Shape
from .named_channels import Size
from .named_channels import Text
from .named_channels import X
from .named_channels import Y
from .channel_collections import Encoding
from .channel_collections import Facet



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

"""
Test that all auto-generated schema objects can be instatiated.
"""

from .. import *


def test_instantiate_AggregateOp():
    obj = AggregateOp()


def test_instantiate_Axis():
    obj = Axis()


def test_instantiate_AxisConfig():
    obj = AxisConfig()


def test_instantiate_AxisOrient():
    obj = AxisOrient()


def test_instantiate_Bin():
    obj = Bin()


def test_instantiate_CellConfig():
    obj = CellConfig()


def test_instantiate_ChannelDefWithLegend():
    obj = ChannelDefWithLegend()


def test_instantiate_Config():
    obj = Config()


def test_instantiate_Data():
    obj = Data()


def test_instantiate_DataFormat():
    obj = DataFormat()


def test_instantiate_Encoding():
    obj = Encoding()


def test_instantiate_ExtendedUnitSpec():
    obj = ExtendedUnitSpec()


def test_instantiate_Facet():
    obj = Facet()


def test_instantiate_FacetConfig():
    obj = FacetConfig()


def test_instantiate_FacetGridConfig():
    obj = FacetGridConfig()


def test_instantiate_FacetScaleConfig():
    obj = FacetScaleConfig()


def test_instantiate_FacetSpec():
    obj = FacetSpec()


def test_instantiate_FieldDef():
    obj = FieldDef()


def test_instantiate_FontStyle():
    obj = FontStyle()


def test_instantiate_FontWeight():
    obj = FontWeight()


def test_instantiate_Formula():
    obj = Formula()


def test_instantiate_HorizontalAlign():
    obj = HorizontalAlign()


def test_instantiate_Interpolate():
    obj = Interpolate()


def test_instantiate_LayerSpec():
    obj = LayerSpec()


def test_instantiate_Legend():
    obj = Legend()


def test_instantiate_LegendConfig():
    obj = LegendConfig()


def test_instantiate_Mark():
    obj = Mark()


def test_instantiate_MarkConfig():
    obj = MarkConfig()


def test_instantiate_NiceTime():
    obj = NiceTime()


def test_instantiate_OrderChannelDef():
    obj = OrderChannelDef()


def test_instantiate_PositionChannelDef():
    obj = PositionChannelDef()


def test_instantiate_Scale():
    obj = Scale()


def test_instantiate_ScaleConfig():
    obj = ScaleConfig()


def test_instantiate_ScaleType():
    obj = ScaleType()


def test_instantiate_Shape():
    obj = Shape()


def test_instantiate_SortField():
    obj = SortField()


def test_instantiate_SortOrder():
    obj = SortOrder()


def test_instantiate_StackOffset():
    obj = StackOffset()


def test_instantiate_TimeUnit():
    obj = TimeUnit()


def test_instantiate_Transform():
    obj = Transform()


def test_instantiate_Type():
    obj = Type()


def test_instantiate_UnitEncoding():
    obj = UnitEncoding()


def test_instantiate_UnitSpec():
    obj = UnitSpec()


def test_instantiate_VerticalAlign():
    obj = VerticalAlign()




# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class AggregateOp(T.Enum):
    """One of ['values', 'count', 'valid', 'missing', 'distinct', 'sum', 'mean', 'average', 'variance', 'variancep', 'stdev', 'stdevp', 'median', 'q1', 'q3', 'modeskew', 'min', 'max', 'argmin', 'argmax']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(AggregateOp, self).__init__(['values', 'count', 'valid', 'missing', 'distinct', 'sum', 'mean', 'average', 'variance', 'variancep', 'stdev', 'stdevp', 'median', 'q1', 'q3', 'modeskew', 'min', 'max', 'argmin', 'argmax'],
                                    default_value=default_value,
                                    **metadata)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class StackOffset(T.Enum):
    """One of ['zero', 'center', 'normalize', 'none']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(StackOffset, self).__init__(['zero', 'center', 'normalize', 'none'],
                                    default_value=default_value,
                                    **metadata)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .config import Config
from .data import Data
from .facet import Facet
from .layerspec import LayerSpec
from .transform import Transform
from .unitspec import UnitSpec


class FacetSpec(BaseObject):
    """Wrapper for Vega-Lite FacetSpec definition.
    
    Attributes
    ----------
    config: Config
        Configuration object.
    data: Data
        An object describing the data source.
    description: Unicode
        An optional description of this mark for commenting purpose.
    facet: Facet
        
    name: Unicode
        Name of the visualization for later reference.
    spec: Union(LayerSpec, UnitSpec)
        
    transform: Transform
        An object describing filter and new field calculation.
    """
    config = T.Instance(Config, allow_none=True, default_value=None, help="""Configuration object.""")
    data = T.Instance(Data, allow_none=True, default_value=None, help="""An object describing the data source.""")
    description = T.Unicode(allow_none=True, default_value=None, help="""An optional description of this mark for commenting purpose.""")
    facet = T.Instance(Facet, allow_none=True, default_value=None)
    name = T.Unicode(allow_none=True, default_value=None, help="""Name of the visualization for later reference.""")
    spec = T.Union([T.Instance(LayerSpec, allow_none=True, default_value=None), T.Instance(UnitSpec, allow_none=True, default_value=None)])
    transform = T.Instance(Transform, allow_none=True, default_value=None, help="""An object describing filter and new field calculation.""")
    
    def __init__(self, config=None, data=None, description=None, facet=None, name=None, spec=None, transform=None, **kwargs):
        kwds = dict(config=config, data=data, description=description, facet=facet, name=name, spec=spec, transform=transform)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(FacetSpec, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .fontstyle import FontStyle
from .fontweight import FontWeight
from .horizontalalign import HorizontalAlign
from .interpolate import Interpolate
from .shape import Shape
from .stackoffset import StackOffset
from .verticalalign import VerticalAlign


class MarkConfig(BaseObject):
    """Wrapper for Vega-Lite MarkConfig definition.
    
    Attributes
    ----------
    align: HorizontalAlign
        The horizontal alignment of the text.
    angle: CFloat
        The rotation angle of the text, in degrees.
    applyColorToBackground: Bool
        Apply color field to background color instead of the text.
    barSize: CFloat
        The size of the bars.
    barThinSize: CFloat
        The size of the bars on continuous scales.
    baseline: VerticalAlign
        The vertical alignment of the text.
    color: Unicode
        Default color.
    dx: CFloat
        The horizontal offset, in pixels, between the text label and its anchor point.
    dy: CFloat
        The vertical offset, in pixels, between the text label and its anchor point.
    fill: Unicode
        Default Fill Color.
    fillOpacity: CFloat
        
    filled: Bool
        Whether the shape\'s color should be used as fill color instead of stroke color.
    font: Unicode
        The typeface to set the text in .
    fontSize: CFloat
        The font size, in pixels.
    fontStyle: FontStyle
        The font style .
    fontWeight: FontWeight
        The font weight .
    format: Unicode
        The formatting pattern for text value.
    interpolate: Interpolate
        The line interpolation method to use.
    lineSize: CFloat
        Size of line mark.
    opacity: CFloat
        
    orient: Unicode
        The orientation of a non-stacked bar, tick, area, and line charts.
    radius: CFloat
        Polar coordinate radial offset, in pixels, of the text label from the origin determined by the x and y properties.
    ruleSize: CFloat
        Size of rule mark.
    shape: Shape
        The symbol shape to use.
    shortTimeLabels: Bool
        Whether month names and weekday names should be abbreviated.
    size: CFloat
        The pixel area each the point.
    stacked: StackOffset
        
    stroke: Unicode
        Default Stroke Color.
    strokeDash: List(CFloat)
        An array of alternating stroke, space lengths for creating dashed or dotted lines.
    strokeDashOffset: CFloat
        The offset (in pixels) into which to begin drawing with the stroke dash array.
    strokeOpacity: CFloat
        
    strokeWidth: CFloat
        
    tension: CFloat
        Depending on the interpolation type, sets the tension parameter.
    text: Unicode
        Placeholder Text.
    theta: CFloat
        Polar coordinate angle, in radians, of the text label from the origin determined by the x and y properties.
    tickSize: CFloat
        The width of the ticks.
    tickThickness: CFloat
        Thickness of the tick mark.
    """
    align = HorizontalAlign(allow_none=True, default_value=None, help="""The horizontal alignment of the text.""")
    angle = T.CFloat(allow_none=True, default_value=None, help="""The rotation angle of the text, in degrees.""")
    applyColorToBackground = T.Bool(allow_none=True, default_value=None, help="""Apply color field to background color instead of the text.""")
    barSize = T.CFloat(allow_none=True, default_value=None, help="""The size of the bars.""")
    barThinSize = T.CFloat(allow_none=True, default_value=None, help="""The size of the bars on continuous scales.""")
    baseline = VerticalAlign(allow_none=True, default_value=None, help="""The vertical alignment of the text.""")
    color = T.Unicode(allow_none=True, default_value=None, help="""Default color.""")
    dx = T.CFloat(allow_none=True, default_value=None, help="""The horizontal offset, in pixels, between the text label and its anchor point.""")
    dy = T.CFloat(allow_none=True, default_value=None, help="""The vertical offset, in pixels, between the text label and its anchor point.""")
    fill = T.Unicode(allow_none=True, default_value=None, help="""Default Fill Color.""")
    fillOpacity = T.CFloat(allow_none=True, default_value=None, min=0, max=1)
    filled = T.Bool(allow_none=True, default_value=None, help="""Whether the shape\'s color should be used as fill color instead of stroke color.""")
    font = T.Unicode(allow_none=True, default_value=None, help="""The typeface to set the text in .""")
    fontSize = T.CFloat(allow_none=True, default_value=None, help="""The font size, in pixels.""")
    fontStyle = FontStyle(allow_none=True, default_value=None, help="""The font style .""")
    fontWeight = FontWeight(allow_none=True, default_value=None, help="""The font weight .""")
    format = T.Unicode(allow_none=True, default_value=None, help="""The formatting pattern for text value.""")
    interpolate = Interpolate(allow_none=True, default_value=None, help="""The line interpolation method to use.""")
    lineSize = T.CFloat(allow_none=True, default_value=None, help="""Size of line mark.""")
    opacity = T.CFloat(allow_none=True, default_value=None, min=0, max=1)
    orient = T.Unicode(allow_none=True, default_value=None, help="""The orientation of a non-stacked bar, tick, area, and line charts.""")
    radius = T.CFloat(allow_none=True, default_value=None, help="""Polar coordinate radial offset, in pixels, of the text label from the origin determined by the x and y properties.""")
    ruleSize = T.CFloat(allow_none=True, default_value=None, help="""Size of rule mark.""")
    shape = Shape(allow_none=True, default_value=None, help="""The symbol shape to use.""")
    shortTimeLabels = T.Bool(allow_none=True, default_value=None, help="""Whether month names and weekday names should be abbreviated.""")
    size = T.CFloat(allow_none=True, default_value=None, help="""The pixel area each the point.""")
    stacked = StackOffset(allow_none=True, default_value=None)
    stroke = T.Unicode(allow_none=True, default_value=None, help="""Default Stroke Color.""")
    strokeDash = T.List(T.CFloat(), allow_none=True, default_value=None, help="""An array of alternating stroke, space lengths for creating dashed or dotted lines.""")
    strokeDashOffset = T.CFloat(allow_none=True, default_value=None, help="""The offset (in pixels) into which to begin drawing with the stroke dash array.""")
    strokeOpacity = T.CFloat(allow_none=True, default_value=None, min=0, max=1)
    strokeWidth = T.CFloat(allow_none=True, default_value=None, min=0)
    tension = T.CFloat(allow_none=True, default_value=None, help="""Depending on the interpolation type, sets the tension parameter.""")
    text = T.Unicode(allow_none=True, default_value=None, help="""Placeholder Text.""")
    theta = T.CFloat(allow_none=True, default_value=None, help="""Polar coordinate angle, in radians, of the text label from the origin determined by the x and y properties.""")
    tickSize = T.CFloat(allow_none=True, default_value=None, help="""The width of the ticks.""")
    tickThickness = T.CFloat(allow_none=True, default_value=None, help="""Thickness of the tick mark.""")
    
    def __init__(self, align=None, angle=None, applyColorToBackground=None, barSize=None, barThinSize=None, baseline=None, color=None, dx=None, dy=None, fill=None, fillOpacity=None, filled=None, font=None, fontSize=None, fontStyle=None, fontWeight=None, format=None, interpolate=None, lineSize=None, opacity=None, orient=None, radius=None, ruleSize=None, shape=None, shortTimeLabels=None, size=None, stacked=None, stroke=None, strokeDash=None, strokeDashOffset=None, strokeOpacity=None, strokeWidth=None, tension=None, text=None, theta=None, tickSize=None, tickThickness=None, **kwargs):
        kwds = dict(align=align, angle=angle, applyColorToBackground=applyColorToBackground, barSize=barSize, barThinSize=barThinSize, baseline=baseline, color=color, dx=dx, dy=dy, fill=fill, fillOpacity=fillOpacity, filled=filled, font=font, fontSize=fontSize, fontStyle=fontStyle, fontWeight=fontWeight, format=format, interpolate=interpolate, lineSize=lineSize, opacity=opacity, orient=orient, radius=radius, ruleSize=ruleSize, shape=shape, shortTimeLabels=shortTimeLabels, size=size, stacked=stacked, stroke=stroke, strokeDash=strokeDash, strokeDashOffset=strokeDashOffset, strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, tension=tension, text=text, theta=theta, tickSize=tickSize, tickThickness=tickThickness)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(MarkConfig, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject


class ScaleConfig(BaseObject):
    """Wrapper for Vega-Lite ScaleConfig definition.
    
    Attributes
    ----------
    bandSize: CFloat
        Default band size for (1) `y` ordinal scale, and (2) `x` ordinal scale when the mark is not `text`.
    barSizeRange: List(CFloat)
        Default range for bar size scale.
    fontSizeRange: List(CFloat)
        Default range for font size scale.
    nominalColorRange: Union(Unicode, List(Unicode))
        Default range for nominal color scale.
    opacity: List(CFloat)
        Default range for opacity.
    padding: CFloat
        Default padding for `x` and `y` ordinal scales.
    pointSizeRange: List(CFloat)
        Default range for bar size scale.
    round: Bool
        If true, rounds numeric output values to integers.
    ruleSizeRange: List(CFloat)
        Default range for rule stroke widths.
    sequentialColorRange: Union(Unicode, List(Unicode))
        Default range for ordinal / continuous color scale.
    shapeRange: Union(Unicode, List(Unicode))
        Default range for shape.
    textBandWidth: CFloat
        Default band width for `x` ordinal scale when is mark is `text`.
    tickSizeRange: List(CFloat)
        Default range for tick spans.
    useRawDomain: Bool
        Uses the source data range as scale domain instead of aggregated data for aggregate axis.
    """
    bandSize = T.CFloat(allow_none=True, default_value=None, min=0, help="""Default band size for (1) `y` ordinal scale, and (2) `x` ordinal scale when the mark is not `text`.""")
    barSizeRange = T.List(T.CFloat(), allow_none=True, default_value=None, help="""Default range for bar size scale.""")
    fontSizeRange = T.List(T.CFloat(), allow_none=True, default_value=None, help="""Default range for font size scale.""")
    nominalColorRange = T.Union([T.Unicode(allow_none=True, default_value=None), T.List(T.Unicode(), allow_none=True, default_value=None)])
    opacity = T.List(T.CFloat(), allow_none=True, default_value=None, help="""Default range for opacity.""")
    padding = T.CFloat(allow_none=True, default_value=None, help="""Default padding for `x` and `y` ordinal scales.""")
    pointSizeRange = T.List(T.CFloat(), allow_none=True, default_value=None, help="""Default range for bar size scale.""")
    round = T.Bool(allow_none=True, default_value=None, help="""If true, rounds numeric output values to integers.""")
    ruleSizeRange = T.List(T.CFloat(), allow_none=True, default_value=None, help="""Default range for rule stroke widths.""")
    sequentialColorRange = T.Union([T.Unicode(allow_none=True, default_value=None), T.List(T.Unicode(), allow_none=True, default_value=None)])
    shapeRange = T.Union([T.Unicode(allow_none=True, default_value=None), T.List(T.Unicode(), allow_none=True, default_value=None)])
    textBandWidth = T.CFloat(allow_none=True, default_value=None, min=0, help="""Default band width for `x` ordinal scale when is mark is `text`.""")
    tickSizeRange = T.List(T.CFloat(), allow_none=True, default_value=None, help="""Default range for tick spans.""")
    useRawDomain = T.Bool(allow_none=True, default_value=None, help="""Uses the source data range as scale domain instead of aggregated data for aggregate axis.""")
    
    def __init__(self, bandSize=None, barSizeRange=None, fontSizeRange=None, nominalColorRange=None, opacity=None, padding=None, pointSizeRange=None, round=None, ruleSizeRange=None, sequentialColorRange=None, shapeRange=None, textBandWidth=None, tickSizeRange=None, useRawDomain=None, **kwargs):
        kwds = dict(bandSize=bandSize, barSizeRange=barSizeRange, fontSizeRange=fontSizeRange, nominalColorRange=nominalColorRange, opacity=opacity, padding=padding, pointSizeRange=pointSizeRange, round=round, ruleSizeRange=ruleSizeRange, sequentialColorRange=sequentialColorRange, shapeRange=shapeRange, textBandWidth=textBandWidth, tickSizeRange=tickSizeRange, useRawDomain=useRawDomain)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(ScaleConfig, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .channeldefwithlegend import ChannelDefWithLegend
from .fielddef import FieldDef
from .orderchanneldef import OrderChannelDef
from .positionchanneldef import PositionChannelDef


class UnitEncoding(BaseObject):
    """Wrapper for Vega-Lite UnitEncoding definition.
    
    Attributes
    ----------
    color: ChannelDefWithLegend
        Color of the marks - either fill or stroke color based on mark type.
    detail: Union(FieldDef, List(FieldDef))
        Additional levels of detail for grouping data in aggregate views and in line and area marks without mapping data to a specific visual channel.
    label: FieldDef
        
    opacity: ChannelDefWithLegend
        Opacity of the marks - either can be a value or in a range.
    order: Union(OrderChannelDef, List(OrderChannelDef))
        Layer order for non-stacked marks, or stack order for stacked marks.
    path: Union(OrderChannelDef, List(OrderChannelDef))
        Order of data points in line marks.
    shape: ChannelDefWithLegend
        The symbol's shape (only for `point` marks).
    size: ChannelDefWithLegend
        Size of the mark.
    text: FieldDef
        Text of the `text` mark.
    x: PositionChannelDef
        X coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).
    y: PositionChannelDef
        Y coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).
    """
    color = T.Instance(ChannelDefWithLegend, allow_none=True, default_value=None, help="""Color of the marks - either fill or stroke color based on mark type.""")
    detail = T.Union([T.Instance(FieldDef, allow_none=True, default_value=None, help="""Interface for any kind of FieldDef."""), T.List(T.Instance(FieldDef, help="""Interface for any kind of FieldDef."""), allow_none=True, default_value=None)])
    label = T.Instance(FieldDef, allow_none=True, default_value=None)
    opacity = T.Instance(ChannelDefWithLegend, allow_none=True, default_value=None, help="""Opacity of the marks - either can be a value or in a range.""")
    order = T.Union([T.Instance(OrderChannelDef, allow_none=True, default_value=None), T.List(T.Instance(OrderChannelDef), allow_none=True, default_value=None)])
    path = T.Union([T.Instance(OrderChannelDef, allow_none=True, default_value=None), T.List(T.Instance(OrderChannelDef), allow_none=True, default_value=None)])
    shape = T.Instance(ChannelDefWithLegend, allow_none=True, default_value=None, help="""The symbol's shape (only for `point` marks).""")
    size = T.Instance(ChannelDefWithLegend, allow_none=True, default_value=None, help="""Size of the mark.""")
    text = T.Instance(FieldDef, allow_none=True, default_value=None, help="""Text of the `text` mark.""")
    x = T.Instance(PositionChannelDef, allow_none=True, default_value=None, help="""X coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).""")
    y = T.Instance(PositionChannelDef, allow_none=True, default_value=None, help="""Y coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).""")
    
    def __init__(self, color=None, detail=None, label=None, opacity=None, order=None, path=None, shape=None, size=None, text=None, x=None, y=None, **kwargs):
        kwds = dict(color=color, detail=detail, label=label, opacity=opacity, order=order, path=path, shape=shape, size=size, text=text, x=x, y=y)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(UnitEncoding, self).__init__(**kwargs)


import json


def to_html(json_dict, template=None, title=None, **kwargs):
    """Embed a Vega-Lite JSON into an HTML document

    Parameters
    ----------
    json_dict : dict
        A dictionary describing the Vega-Lite specification.
    template : string
        The HTML template to use. This should have a format method, which
        accepts a "spec" and "title" argument. Note that a standard Python
        format string meets these requirements.
        By default, uses DEFAULT_TEMPLATE.
    title: string
        The title to use in the document. Default is "Vega-Lite Chart"

    Returns
    -------
    html : string
        A string of HTML representing the chart
    """
    if template is None:
        template = DEFAULT_TEMPLATE
    if title is None:
        title = "Vega-Lite Chart"
    spec = json.dumps(json_dict, indent=4)
    return template.format(spec=spec, title=title, **kwargs)


DEFAULT_TEMPLATE = """
<!DOCTYPE html>
<head>
  <title>{title}</title>
  <meta charset="utf-8">

  <script src="//d3js.org/d3.v3.min.js"></script>
  <script src="//vega.github.io/vega/vega.js"></script>
  <script src="//vega.github.io/vega-lite/vega-lite.js"></script>
  <script src="//vega.github.io/vega-editor/vendor/vega-embed.js" charset="utf-8"></script>

  <style media="screen">
    /* Add space between vega-embed links  */
    .vega-actions a {{
      margin-right: 5px;
    }}
  </style>
</head>
<body>
  <!-- Container for the visualization -->
  <div id="vis"></div>

  <script>
  var vlSpec = {spec}

  var embedSpec = {{
    mode: "vega-lite",  // Instruct Vega-Embed to use the Vega-Lite compiler
    spec: vlSpec
  }};

  // Embed the visualization in the container with id `vis`
  vg.embed("#vis", embedSpec, function(error, result) {{
    // Callback receiving the View instance and parsed Vega spec
    // result.view is the View, which resides under the '#vis' element
  }});
  </script>
</body>
</html>
"""



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .aggregateop import AggregateOp
from .bin import Bin
from .sortorder import SortOrder
from .timeunit import TimeUnit
from .type import Type


class OrderChannelDef(BaseObject):
    """Wrapper for Vega-Lite OrderChannelDef definition.
    
    Attributes
    ----------
    aggregate: AggregateOp
        Aggregation function for the field .
    bin: Union(Bool, Bin)
        Flag for binning a `quantitative` field, or a bin property object for binning parameters.
    field: Unicode
        Name of the field from which to pull a data value.
    sort: SortOrder
        
    timeUnit: TimeUnit
        Time unit for a `temporal` field .
    title: Unicode
        Title for axis or legend.
    type: Type
        The encoded field's type of measurement.
    value: Union(CFloat, Unicode, Bool)
        A constant value in visual domain.
    """
    aggregate = AggregateOp(allow_none=True, default_value=None, help="""Aggregation function for the field .""")
    bin = T.Union([T.Bool(allow_none=True, default_value=None), T.Instance(Bin, allow_none=True, default_value=None, help="""Binning properties or boolean flag for determining whether to bin data or not.""")])
    field = T.Unicode(allow_none=True, default_value=None, help="""Name of the field from which to pull a data value.""")
    sort = SortOrder(allow_none=True, default_value=None)
    timeUnit = TimeUnit(allow_none=True, default_value=None, help="""Time unit for a `temporal` field .""")
    title = T.Unicode(allow_none=True, default_value=None, help="""Title for axis or legend.""")
    type = Type(allow_none=True, default_value=None, help="""The encoded field's type of measurement.""")
    value = T.Union([T.CFloat(allow_none=True, default_value=None), T.Unicode(allow_none=True, default_value=None), T.Bool(allow_none=True, default_value=None)])
    
    def __init__(self, aggregate=None, bin=None, field=None, sort=None, timeUnit=None, title=None, type=None, value=None, **kwargs):
        kwds = dict(aggregate=aggregate, bin=bin, field=field, sort=sort, timeUnit=timeUnit, title=title, type=type, value=value)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(OrderChannelDef, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class Shape(T.Enum):
    """One of ['circle', 'square', 'cross', 'diamond', 'triangle-up', 'triangle-down']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(Shape, self).__init__(['circle', 'square', 'cross', 'diamond', 'triangle-up', 'triangle-down'],
                                    default_value=default_value,
                                    **metadata)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .dataformat import DataFormat


class Data(BaseObject):
    """Wrapper for Vega-Lite Data definition.
    
    Attributes
    ----------
    formatType: DataFormat
        
    url: Unicode
        A URL from which to load the data set.
    values: List(Any)
        Pass array of objects instead of a url to a file.
    """
    formatType = DataFormat(allow_none=True, default_value=None)
    url = T.Unicode(allow_none=True, default_value=None, help="""A URL from which to load the data set.""")
    values = T.List(T.Any(), allow_none=True, default_value=None, help="""Pass array of objects instead of a url to a file.""")
    
    def __init__(self, formatType=None, url=None, values=None, **kwargs):
        kwds = dict(formatType=formatType, url=url, values=values)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Data, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .axisconfig import AxisConfig
from .cellconfig import CellConfig
from .facetconfig import FacetConfig
from .legendconfig import LegendConfig
from .markconfig import MarkConfig
from .scaleconfig import ScaleConfig


class Config(BaseObject):
    """Wrapper for Vega-Lite Config definition.
    
    Attributes
    ----------
    axis: AxisConfig
        Axis Config.
    background: Unicode
        CSS color property to use as background of visualization.
    cell: CellConfig
        Cell Config.
    facet: FacetConfig
        Facet Config.
    legend: LegendConfig
        Legend Config.
    mark: MarkConfig
        Mark Config.
    numberFormat: Unicode
        D3 Number format for axis labels and text tables.
    scale: ScaleConfig
        Scale Config.
    timeFormat: Unicode
        Default datetime format for axis and legend labels.
    viewport: CFloat
        The width and height of the on-screen viewport, in pixels.
    """
    axis = T.Instance(AxisConfig, allow_none=True, default_value=None, help="""Axis Config.""")
    background = T.Unicode(allow_none=True, default_value=None, help="""CSS color property to use as background of visualization.""")
    cell = T.Instance(CellConfig, allow_none=True, default_value=None, help="""Cell Config.""")
    facet = T.Instance(FacetConfig, allow_none=True, default_value=None, help="""Facet Config.""")
    legend = T.Instance(LegendConfig, allow_none=True, default_value=None, help="""Legend Config.""")
    mark = T.Instance(MarkConfig, allow_none=True, default_value=None, help="""Mark Config.""")
    numberFormat = T.Unicode(allow_none=True, default_value=None, help="""D3 Number format for axis labels and text tables.""")
    scale = T.Instance(ScaleConfig, allow_none=True, default_value=None, help="""Scale Config.""")
    timeFormat = T.Unicode(allow_none=True, default_value=None, help="""Default datetime format for axis and legend labels.""")
    viewport = T.CFloat(allow_none=True, default_value=None, help="""The width and height of the on-screen viewport, in pixels.""")
    
    def __init__(self, axis=None, background=None, cell=None, facet=None, legend=None, mark=None, numberFormat=None, scale=None, timeFormat=None, viewport=None, **kwargs):
        kwds = dict(axis=axis, background=background, cell=cell, facet=facet, legend=legend, mark=mark, numberFormat=numberFormat, scale=scale, timeFormat=timeFormat, viewport=viewport)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Config, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .aggregateop import AggregateOp
from .axis import Axis
from .bin import Bin
from .scale import Scale
from .sortfield import SortField
from .sortorder import SortOrder
from .timeunit import TimeUnit
from .type import Type


class PositionChannelDef(BaseObject):
    """Wrapper for Vega-Lite PositionChannelDef definition.
    
    Attributes
    ----------
    aggregate: AggregateOp
        Aggregation function for the field .
    axis: Union(Bool, Axis)
        
    bin: Union(Bool, Bin)
        Flag for binning a `quantitative` field, or a bin property object for binning parameters.
    field: Unicode
        Name of the field from which to pull a data value.
    scale: Scale
        
    sort: Union(SortField, SortOrder)
        
    timeUnit: TimeUnit
        Time unit for a `temporal` field .
    title: Unicode
        Title for axis or legend.
    type: Type
        The encoded field's type of measurement.
    value: Union(CFloat, Unicode, Bool)
        A constant value in visual domain.
    """
    aggregate = AggregateOp(allow_none=True, default_value=None, help="""Aggregation function for the field .""")
    axis = T.Union([T.Bool(allow_none=True, default_value=None), T.Instance(Axis, allow_none=True, default_value=None)])
    bin = T.Union([T.Bool(allow_none=True, default_value=None), T.Instance(Bin, allow_none=True, default_value=None, help="""Binning properties or boolean flag for determining whether to bin data or not.""")])
    field = T.Unicode(allow_none=True, default_value=None, help="""Name of the field from which to pull a data value.""")
    scale = T.Instance(Scale, allow_none=True, default_value=None)
    sort = T.Union([T.Instance(SortField, allow_none=True, default_value=None), SortOrder(allow_none=True, default_value=None)])
    timeUnit = TimeUnit(allow_none=True, default_value=None, help="""Time unit for a `temporal` field .""")
    title = T.Unicode(allow_none=True, default_value=None, help="""Title for axis or legend.""")
    type = Type(allow_none=True, default_value=None, help="""The encoded field's type of measurement.""")
    value = T.Union([T.CFloat(allow_none=True, default_value=None), T.Unicode(allow_none=True, default_value=None), T.Bool(allow_none=True, default_value=None)])
    
    def __init__(self, aggregate=None, axis=None, bin=None, field=None, scale=None, sort=None, timeUnit=None, title=None, type=None, value=None, **kwargs):
        kwds = dict(aggregate=aggregate, axis=axis, bin=bin, field=field, scale=scale, sort=sort, timeUnit=timeUnit, title=title, type=type, value=value)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(PositionChannelDef, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .formula import Formula


class Transform(BaseObject):
    """Wrapper for Vega-Lite Transform definition.
    
    Attributes
    ----------
    calculate: List(Formula)
        Calculate new field(s) using the provided expresssion(s).
    filter: Unicode
        A string containing the filter Vega expression.
    filterNull: Bool
        Filter null values from the data.
    """
    calculate = T.List(T.Instance(Formula, help="""Formula object for calculate."""), allow_none=True, default_value=None, help="""Calculate new field(s) using the provided expresssion(s).""")
    filter = T.Unicode(allow_none=True, default_value=None, help="""A string containing the filter Vega expression.""")
    filterNull = T.Bool(allow_none=True, default_value=None, help="""Filter null values from the data.""")
    
    def __init__(self, calculate=None, filter=None, filterNull=None, **kwargs):
        kwds = dict(calculate=calculate, filter=filter, filterNull=filterNull)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Transform, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
import pandas as pd

from ...utils import parse_shorthand, infer_vegalite_type

from .._interface import ChannelDefWithLegend, FieldDef, OrderChannelDef, PositionChannelDef


class PositionChannel(PositionChannelDef):
    """Wrapper for Vega-Lite PositionChannelDef definition.
    
    Attributes
    ----------
    shorthand: Unicode
        A shorthand description of the channel
    aggregate: AggregateOp
        Aggregation function for the field .
    axis: Union(Bool, Axis)
        
    bin: Union(Bool, Bin)
        Flag for binning a `quantitative` field, or a bin property object for binning parameters.
    field: Unicode
        Name of the field from which to pull a data value.
    scale: Scale
        
    sort: Union(SortField, SortOrder)
        
    timeUnit: TimeUnit
        Time unit for a `temporal` field .
    title: Unicode
        Title for axis or legend.
    type: Union(Type, Unicode)
        The encoded field's type of measurement.
    value: Union(CFloat, Unicode, Bool)
        A constant value in visual domain.
    """
    # Traitlets
    shorthand = T.Unicode('')
    skip = ['shorthand']

    # Class Methods
    def __init__(self, shorthand='', aggregate=None, axis=None, bin=None, field=None, scale=None, sort=None, timeUnit=None, title=None, type=None, value=None, **kwargs):
        kwargs['shorthand'] = shorthand
        kwds = dict(aggregate=aggregate, axis=axis, bin=bin, field=field, scale=scale, sort=sort, timeUnit=timeUnit, title=title, type=type, value=value)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(PositionChannel, self).__init__(**kwargs)

    def _finalize(self, **kwargs):
        """Finalize object: this involves inferring types if necessary"""
        # parse the shorthand to extract the field, type, and aggregate
        for key, val in parse_shorthand(self.shorthand).items():
            setattr(self, key, val)

        # infer the type if not already specified
        if self.type is None:
            data = kwargs.get('data', None)
            if isinstance(data, pd.DataFrame) and self.field in data:
                self.type = infer_vegalite_type(data[self.field])

        super(PositionChannel, self)._finalize(**kwargs)


class ChannelWithLegend(ChannelDefWithLegend):
    """Wrapper for Vega-Lite ChannelDefWithLegend definition.
    
    Attributes
    ----------
    shorthand: Unicode
        A shorthand description of the channel
    aggregate: AggregateOp
        Aggregation function for the field .
    bin: Union(Bool, Bin)
        Flag for binning a `quantitative` field, or a bin property object for binning parameters.
    field: Unicode
        Name of the field from which to pull a data value.
    legend: Legend
        
    scale: Scale
        
    sort: Union(SortField, SortOrder)
        
    timeUnit: TimeUnit
        Time unit for a `temporal` field .
    title: Unicode
        Title for axis or legend.
    type: Union(Type, Unicode)
        The encoded field's type of measurement.
    value: Union(CFloat, Unicode, Bool)
        A constant value in visual domain.
    """
    # Traitlets
    shorthand = T.Unicode('')
    skip = ['shorthand']

    # Class Methods
    def __init__(self, shorthand='', aggregate=None, bin=None, field=None, legend=None, scale=None, sort=None, timeUnit=None, title=None, type=None, value=None, **kwargs):
        kwargs['shorthand'] = shorthand
        kwds = dict(aggregate=aggregate, bin=bin, field=field, legend=legend, scale=scale, sort=sort, timeUnit=timeUnit, title=title, type=type, value=value)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(ChannelWithLegend, self).__init__(**kwargs)

    def _finalize(self, **kwargs):
        """Finalize object: this involves inferring types if necessary"""
        # parse the shorthand to extract the field, type, and aggregate
        for key, val in parse_shorthand(self.shorthand).items():
            setattr(self, key, val)

        # infer the type if not already specified
        if self.type is None:
            data = kwargs.get('data', None)
            if isinstance(data, pd.DataFrame) and self.field in data:
                self.type = infer_vegalite_type(data[self.field])

        super(ChannelWithLegend, self)._finalize(**kwargs)


class Field(FieldDef):
    """Wrapper for Vega-Lite FieldDef definition.
    
    Attributes
    ----------
    shorthand: Unicode
        A shorthand description of the channel
    aggregate: AggregateOp
        Aggregation function for the field .
    bin: Union(Bool, Bin)
        Flag for binning a `quantitative` field, or a bin property object for binning parameters.
    field: Unicode
        Name of the field from which to pull a data value.
    timeUnit: TimeUnit
        Time unit for a `temporal` field .
    title: Unicode
        Title for axis or legend.
    type: Union(Type, Unicode)
        The encoded field's type of measurement.
    value: Union(CFloat, Unicode, Bool)
        A constant value in visual domain.
    """
    # Traitlets
    shorthand = T.Unicode('')
    skip = ['shorthand']

    # Class Methods
    def __init__(self, shorthand='', aggregate=None, bin=None, field=None, timeUnit=None, title=None, type=None, value=None, **kwargs):
        kwargs['shorthand'] = shorthand
        kwds = dict(aggregate=aggregate, bin=bin, field=field, timeUnit=timeUnit, title=title, type=type, value=value)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Field, self).__init__(**kwargs)

    def _finalize(self, **kwargs):
        """Finalize object: this involves inferring types if necessary"""
        # parse the shorthand to extract the field, type, and aggregate
        for key, val in parse_shorthand(self.shorthand).items():
            setattr(self, key, val)

        # infer the type if not already specified
        if self.type is None:
            data = kwargs.get('data', None)
            if isinstance(data, pd.DataFrame) and self.field in data:
                self.type = infer_vegalite_type(data[self.field])

        super(Field, self)._finalize(**kwargs)


class OrderChannel(OrderChannelDef):
    """Wrapper for Vega-Lite OrderChannelDef definition.
    
    Attributes
    ----------
    shorthand: Unicode
        A shorthand description of the channel
    aggregate: AggregateOp
        Aggregation function for the field .
    bin: Union(Bool, Bin)
        Flag for binning a `quantitative` field, or a bin property object for binning parameters.
    field: Unicode
        Name of the field from which to pull a data value.
    sort: SortOrder
        
    timeUnit: TimeUnit
        Time unit for a `temporal` field .
    title: Unicode
        Title for axis or legend.
    type: Union(Type, Unicode)
        The encoded field's type of measurement.
    value: Union(CFloat, Unicode, Bool)
        A constant value in visual domain.
    """
    # Traitlets
    shorthand = T.Unicode('')
    skip = ['shorthand']

    # Class Methods
    def __init__(self, shorthand='', aggregate=None, bin=None, field=None, sort=None, timeUnit=None, title=None, type=None, value=None, **kwargs):
        kwargs['shorthand'] = shorthand
        kwds = dict(aggregate=aggregate, bin=bin, field=field, sort=sort, timeUnit=timeUnit, title=title, type=type, value=value)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(OrderChannel, self).__init__(**kwargs)

    def _finalize(self, **kwargs):
        """Finalize object: this involves inferring types if necessary"""
        # parse the shorthand to extract the field, type, and aggregate
        for key, val in parse_shorthand(self.shorthand).items():
            setattr(self, key, val)

        # infer the type if not already specified
        if self.type is None:
            data = kwargs.get('data', None)
            if isinstance(data, pd.DataFrame) and self.field in data:
                self.type = infer_vegalite_type(data[self.field])

        super(OrderChannel, self)._finalize(**kwargs)





# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class Interpolate(T.Enum):
    """One of ['linear', 'linear-closed', 'step', 'step-before', 'step-after', 'basis', 'basis-open', 'basis-closed', 'cardinal', 'cardinal-open', 'cardinal-closed', 'bundle', 'monotone']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(Interpolate, self).__init__(['linear', 'linear-closed', 'step', 'step-before', 'step-after', 'basis', 'basis-open', 'basis-closed', 'cardinal', 'cardinal-open', 'cardinal-closed', 'bundle', 'monotone'],
                                    default_value=default_value,
                                    **metadata)


import os
import pkgutil
import json

import pytest

from .. import iter_examples
from ... import examples
from ... import *


@pytest.mark.parametrize('example', iter_examples())
def test_json_examples_round_trip(example):
    """
    Test that Altair correctly round-trips JSON with to_dict() and to_altair()
    """
    filename, json_dict = example

    v = load_vegalite_spec(json_dict)
    v_dict = v.to_dict()
    assert v_dict == json_dict

    v2 = eval(v.to_altair())
    assert v_dict == v2.to_dict()



"""
Main API for Vega-lite spec generation.

DSL mapping Vega types to IPython traitlets.
"""
import traitlets as T
import pandas as pd

from .utils import visitors
from .utils._py3k_compat import string_types

from . import schema

from .schema import AggregateOp
from .schema import AxisConfig
from .schema import AxisOrient
from .schema import Axis
from .schema import Bin
from .schema import CellConfig
from .schema import Config
from .schema import Data
from .schema import DataFormat
from .schema import FacetConfig
from .schema import FacetGridConfig
from .schema import FacetScaleConfig
from .schema import FontStyle
from .schema import FontWeight
from .schema import Formula
from .schema import HorizontalAlign
from .schema import LegendConfig
from .schema import Legend
from .schema import MarkConfig
from .schema import NiceTime
from .schema import Scale
from .schema import ScaleConfig
from .schema import SortField
from .schema import SortOrder
from .schema import StackOffset
from .schema import TimeUnit
from .schema import Transform
from .schema import UnitSpec
from .schema import UnitEncoding
from .schema import VerticalAlign

#*************************************************************************
# Channel Aliases
#*************************************************************************
from .schema import X, Y, Row, Column, Color, Size, Shape, Text, Label, Detail, Opacity, Order, Path
from .schema import Encoding, Facet


#*************************************************************************
# Loading the spec
#*************************************************************************
def load_vegalite_spec(spec):
    """Load a Vega-Lite spec and return an Altair object.

    The spec should be in the form of a Python dictionary.
    """
    if 'layers' in spec:
        return LayeredChart.from_dict(spec)
    elif 'facet' in spec:
        return FacetedChart.from_dict(spec)
    else:
        return Chart.from_dict(spec)


def use_signature(Obj):
    """Apply call signature and documentation of Obj to the decorated method"""
    def decorate(f):
        # call-signature of f is exposed via __wrapped__.
        # we want it to mimic Obj.__init__
        f.__wrapped__ = Obj.__init__

        # Supplement the docstring of f with information from Obj
        f.__doc__ += Obj.__doc__[Obj.__doc__.index('\n'):]
        return f
    return decorate


#*************************************************************************
# Top-level Objects
#*************************************************************************
class TopLevelMixin(object):
    def to_html(self, template=None, title=None, **kwargs):
        """Emit a stand-alone HTML document containing this chart.

        Parameters
        ----------
        template : string
            The HTML template to use. This should have a format method, which
            accepts a "spec" and "title" argument. Note that a standard Python
            format string meets these requirements.
            By default, uses altair.utils.html.DEFAULT_TEMPLATE.
        title : string
            The title to use in the document. Default is "Vega-Lite Chart"
        kwargs :
            additional keywords to be passed to the template

        Returns
        -------
        html : string
            A string of HTML representing the chart
        """
        from .utils.html import to_html
        return to_html(self.to_dict(), template=template, title=title)

    def _to_code(self, data=None):
        """Emit the CodeGen object used to export this chart to Python code."""
        # do not call _finalize(), as we want the output code
        # to reflect the exact input
        return visitors.ToCode().visit(self, data=data)

    def to_altair(self, data=None):
        """Emit the Python code as a string required to created this Chart."""
        return str(self._to_code(data=data))

    # transform method
    @use_signature(schema.Transform)
    def transform_data(self, *args, **kwargs):
        """Set the data transform by keyword args."""
        return self.update_subtraits('transform', *args, **kwargs)

    # Configuration methods
    @use_signature(schema.Config)
    def configure(self, *args, **kwargs):
        """Set chart configuration"""
        return self.update_subtraits('config', *args, **kwargs)

    @use_signature(AxisConfig)
    def configure_axis(self, *args, **kwargs):
        """Configure the chart's axes by keyword args."""
        return self.update_subtraits(('config', 'axis'), *args, **kwargs)

    @use_signature(CellConfig)
    def configure_cell(self, *args, **kwargs):
        """Configure the chart's cell's by keyword args."""
        return self.update_subtraits(('config', 'cell'), *args, **kwargs)

    @use_signature(LegendConfig)
    def configure_legend(self, *args, **kwargs):
        """Configure the chart's legend by keyword args."""
        return self.update_subtraits(('config', 'legend'), *args, **kwargs)

    @use_signature(MarkConfig)
    def configure_mark(self, *args, **kwargs):
        """Configure the chart's marks by keyword args."""
        return self.update_subtraits(('config', 'mark'), *args, **kwargs)

    @use_signature(ScaleConfig)
    def configure_scale(self, *args, **kwargs):
        """Configure the chart's scales by keyword args."""
        return self.update_subtraits(('config', 'scale'), *args, **kwargs)

    @use_signature(FacetConfig)
    def configure_facet(self, *args, **kwargs):
        """Configure the chart's scales by keyword args."""
        return self.update_subtraits(('config', 'facet'), *args, **kwargs)

    @use_signature(AxisConfig)
    def configure_facet_axis(self, *args, **kwargs):
        """Configure the facet's axes by keyword args."""
        return self.update_subtraits(('config', 'facet', 'axis'),
                                     *args, **kwargs)

    @use_signature(CellConfig)
    def configure_facet_cell(self, *args, **kwargs):
        """Configure the facet's cells by keyword args."""
        return self.update_subtraits(('config', 'facet', 'cell'),
                                     *args, **kwargs)

    @use_signature(FacetGridConfig)
    def configure_facet_grid(self, *args, **kwargs):
        """Configure the facet's grid by keyword args."""
        return self.update_subtraits(('config', 'facet', 'grid'),
                                     *args, **kwargs)

    @use_signature(FacetScaleConfig)
    def configure_facet_scale(self, *args, **kwargs):
        """Configure the facet's scales by keyword args."""
        return self.update_subtraits(('config', 'facet', 'scale'),
                                     *args, **kwargs)

    # Display related methods
    def _ipython_display_(self):
        from IPython.display import display
        from vega import VegaLite
        display(VegaLite(self.to_dict()))

    def display(self):
        from IPython.display import display
        display(self)


class Chart(schema.ExtendedUnitSpec, TopLevelMixin):
    _data = None

    name = T.Unicode()
    description = T.Unicode()
    transform = T.Instance(schema.Transform, default_value=None, allow_none=True)
    mark = T.Enum(schema.Mark().values, default_value='point')
    encoding = T.Instance(Encoding, default_value=None, allow_none=True)
    config = T.Instance(schema.Config, allow_none=True)

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, new):
        if isinstance(new, string_types):
            self._data = Data(url=new)
        elif (isinstance(new, pd.DataFrame) or isinstance(new, Data) or new is None):
            self._data = new
        else:
            raise TypeError('Expected DataFrame or altair.Data, got: {0}'.format(new))

    skip = ['data', '_data']

    def __init__(self, data=None, **kwargs):
        super(Chart, self).__init__(**kwargs)
        self.data = data

    def __dir__(self):
        base = super(Chart, self).__dir__()
        methods = [
            'to_altair', 'display',
            'configure', 'configure_axis', 'configure_cell',
            'configure_legend', 'configure_mark', 'configure_scale',
            'configure_facet_axis', 'configure_facet_cell',
            'configure_facet_grid', 'configure_facet_scale',
            'transform_data',
            'mark_area', 'mark_bar', 'mark_line', 'mark_point', 'mark_rule',
            'mark_text', 'mark_tick', 'mark_circle', 'mark_square',
            'encode',
        ]
        return base+methods

    @use_signature(MarkConfig)
    def mark_area(self, *args, **kwargs):
        """Set the mark to 'area' and optionally specify mark properties"""
        self.mark = 'area'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_bar(self, *args, **kwargs):
        """Set the mark to 'bar' and optionally specify mark properties"""
        self.mark = 'bar'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_line(self, *args, **kwargs):
        """Set the mark to 'line' and optionally specify mark properties"""
        self.mark = 'line'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_point(self, *args, **kwargs):
        """Set the mark to 'point' and optionally specify mark properties"""
        self.mark = 'point'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_rule(self, *args, **kwargs):
        """Set the mark to 'rule' and optionally specify mark properties"""
        self.mark = 'rule'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_text(self, *args, **kwargs):
        """Set the mark to 'text' and optionally specify mark properties"""
        self.mark = 'text'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_tick(self, *args, **kwargs):
        """Set the mark to 'tick' and optionally specify mark properties"""
        self.mark = 'tick'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_circle(self, *args, **kwargs):
        """Set the mark to 'circle' and optionally specify mark properties"""
        self.mark = 'circle'
        return self.configure_mark(*args, **kwargs)

    @use_signature(MarkConfig)
    def mark_square(self, *args, **kwargs):
        """Set the mark to 'square' and optionally specify mark properties"""
        self.mark = 'square'
        return self.configure_mark(*args, **kwargs)

    @use_signature(Encoding)
    def encode(self, *args, **kwargs):
        """Define the encoding for the Chart."""
        return self.update_subtraits('encoding', *args, **kwargs)

    def _finalize(self, **kwargs):
        # data comes from wrappers, but self.data overrides this if defined
        if self.data is not None:
            kwargs['data'] = self.data
        super(Chart, self)._finalize(**kwargs)

    def __add__(self, other):
        if isinstance(other, Chart):
            lc = LayeredChart()
            lc += self
            lc += other
            return lc
        else:
            raise TypeError('Can only add Charts/LayeredChart to Chart')
    

class LayeredChart(schema.LayerSpec, TopLevelMixin):
    _data = None

    name = T.Unicode()
    description = T.Unicode()
    layers = T.List(T.Instance(Chart), allow_none=True, default_value=None)
    transform = T.Instance(schema.Transform, allow_none=True, default_value=None)
    config = T.Instance(schema.Config, allow_none=True, default_value=None)

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, new):
        if isinstance(new, string_types):
            self._data = Data(url=new)
        elif (isinstance(new, pd.DataFrame) or isinstance(new, Data) or new is None):
            self._data = new
        else:
            raise TypeError('Expected DataFrame or altair.Data, got: {0}'.format(new))

    skip = ['data', '_data']

    def __init__(self, data=None, **kwargs):
        super(LayeredChart, self).__init__(**kwargs)
        self.data = data

    def __dir__(self):
        base = super(Chart, self).__dir__()
        methods = [
            'to_dict', 'from_dict', 'to_altair', 'display',
            'configure', 'configure_axis', 'configure_cell',
            'configure_legend', 'configure_mark', 'configure_scale',
            'configure_facet_axis', 'configure_facet_cell',
            'configure_facet_grid', 'configure_facet_scale',
            'transform_data',
            'set_layers',
        ]
        return base+methods

    def set_layers(self, *layers):
        self.layers = list(layers)
        return self

    def _finalize(self, **kwargs):
        # data comes from wrappers, but self.data overrides this if defined
        if self.data is not None:
            kwargs['data'] = self.data
        super(LayeredChart, self)._finalize(**kwargs)

    def __iadd__(self, layer):
        if self.layers is None:
            self.layers = [layer]
        else:
            self.layers = self.layers + [layer]
        return self
    

class FacetedChart(schema.FacetSpec, TopLevelMixin):
    _data = None

    name = T.Unicode()
    description = T.Unicode()
    facet = T.Instance(Facet, allow_none=True, default_value=None)
    spec = T.Union([T.Instance(LayeredChart), T.Instance(Chart)], allow_none=True, default_value=None)
    transform = T.Instance(schema.Transform, allow_none=True, default_value=None)
    config = T.Instance(schema.Config, allow_none=True, default_value=None)

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, new):
        if isinstance(new, string_types):
            self._data = Data(url=new)
        elif (isinstance(new, pd.DataFrame) or isinstance(new, Data) or new is None):
            self._data = new
        else:
            raise TypeError('Expected DataFrame or altair.Data, got: {0}'.format(new))

    skip = ['data', '_data']

    def __init__(self, data=None, **kwargs):
        super(FacetedChart, self).__init__(**kwargs)
        self.data = data

    def __dir__(self):
        base = super(Chart, self).__dir__()
        methods = [
            'to_dict', 'from_dict', 'to_altair', 'display',
            'configure', 'configure_axis', 'configure_cell',
            'configure_legend', 'configure_mark', 'configure_scale',
            'configure_facet_axis', 'configure_facet_cell',
            'configure_facet_grid', 'configure_facet_scale',
            'transform_data',
            'set_facet',
        ]
        return base + methods

    @use_signature(Facet)
    def set_facet(self, *args, **kwargs):
        """Define the facet encoding for the Chart."""
        return self.update_subtraits('facet', *args, **kwargs)

    def _finalize(self, **kwargs):
        # data comes from wrappers, but self.data overrides this if defined
        if self.data is not None:
            kwargs['data'] = self.data
        super(FacetedChart, self)._finalize(**kwargs)



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject


class FacetGridConfig(BaseObject):
    """Wrapper for Vega-Lite FacetGridConfig definition.
    
    Attributes
    ----------
    color: Unicode
        
    offset: CFloat
        
    opacity: CFloat
        
    """
    color = T.Unicode(allow_none=True, default_value=None)
    offset = T.CFloat(allow_none=True, default_value=None)
    opacity = T.CFloat(allow_none=True, default_value=None)
    
    def __init__(self, color=None, offset=None, opacity=None, **kwargs):
        kwds = dict(color=color, offset=offset, opacity=opacity)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(FacetGridConfig, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject


class LegendConfig(BaseObject):
    """Wrapper for Vega-Lite LegendConfig definition.
    
    Attributes
    ----------
    gradientHeight: CFloat
        The height of the gradient, in pixels.
    gradientStrokeColor: Unicode
        The color of the gradient stroke, can be in hex color code or regular color name.
    gradientStrokeWidth: CFloat
        The width of the gradient stroke, in pixels.
    gradientWidth: CFloat
        The width of the gradient, in pixels.
    labelAlign: Unicode
        The alignment of the legend label, can be left, middle or right.
    labelBaseline: Unicode
        The position of the baseline of legend label, can be top, middle or bottom.
    labelColor: Unicode
        The color of the legend label, can be in hex color code or regular color name.
    labelFont: Unicode
        The font of the lengend label.
    labelFontSize: CFloat
        The font size of lengend lable.
    labelOffset: CFloat
        The offset of the legend label.
    margin: CFloat
        The margin around the legend, in pixels.
    offset: CFloat
        The offset, in pixels, by which to displace the legend from the edge of the enclosing group or data rectangle.
    orient: Unicode
        The orientation of the legend.
    padding: CFloat
        The padding, in pixels, between the lengend and axis.
    properties: Any
        Optional mark property definitions for custom legend styling.
    shortTimeLabels: Bool
        Whether month names and weekday names should be abbreviated.
    symbolColor: Unicode
        The color of the legend symbol,.
    symbolShape: Unicode
        The shape of the legend symbol, can be the 'circle', 'square', 'cross', 'diamond', 'triangle-up', 'triangle-down'.
    symbolSize: CFloat
        The size of the lengend symbol, in pixels.
    symbolStrokeWidth: CFloat
        The width of the symbol's stroke.
    titleColor: Unicode
        Optional mark property definitions for custom legend styling.
    titleFont: Unicode
        The font of the legend title.
    titleFontSize: CFloat
        The font size of the legend title.
    titleFontWeight: Unicode
        The font weight of the legend title.
    """
    gradientHeight = T.CFloat(allow_none=True, default_value=None, help="""The height of the gradient, in pixels.""")
    gradientStrokeColor = T.Unicode(allow_none=True, default_value=None, help="""The color of the gradient stroke, can be in hex color code or regular color name.""")
    gradientStrokeWidth = T.CFloat(allow_none=True, default_value=None, help="""The width of the gradient stroke, in pixels.""")
    gradientWidth = T.CFloat(allow_none=True, default_value=None, help="""The width of the gradient, in pixels.""")
    labelAlign = T.Unicode(allow_none=True, default_value=None, help="""The alignment of the legend label, can be left, middle or right.""")
    labelBaseline = T.Unicode(allow_none=True, default_value=None, help="""The position of the baseline of legend label, can be top, middle or bottom.""")
    labelColor = T.Unicode(allow_none=True, default_value=None, help="""The color of the legend label, can be in hex color code or regular color name.""")
    labelFont = T.Unicode(allow_none=True, default_value=None, help="""The font of the lengend label.""")
    labelFontSize = T.CFloat(allow_none=True, default_value=None, help="""The font size of lengend lable.""")
    labelOffset = T.CFloat(allow_none=True, default_value=None, help="""The offset of the legend label.""")
    margin = T.CFloat(allow_none=True, default_value=None, help="""The margin around the legend, in pixels.""")
    offset = T.CFloat(allow_none=True, default_value=None, help="""The offset, in pixels, by which to displace the legend from the edge of the enclosing group or data rectangle.""")
    orient = T.Unicode(allow_none=True, default_value=None, help="""The orientation of the legend.""")
    padding = T.CFloat(allow_none=True, default_value=None, help="""The padding, in pixels, between the lengend and axis.""")
    properties = T.Any(allow_none=True, default_value=None, help="""Optional mark property definitions for custom legend styling.""")
    shortTimeLabels = T.Bool(allow_none=True, default_value=None, help="""Whether month names and weekday names should be abbreviated.""")
    symbolColor = T.Unicode(allow_none=True, default_value=None, help="""The color of the legend symbol,.""")
    symbolShape = T.Unicode(allow_none=True, default_value=None, help="""The shape of the legend symbol, can be the 'circle', 'square', 'cross', 'diamond', 'triangle-up', 'triangle-down'.""")
    symbolSize = T.CFloat(allow_none=True, default_value=None, help="""The size of the lengend symbol, in pixels.""")
    symbolStrokeWidth = T.CFloat(allow_none=True, default_value=None, help="""The width of the symbol's stroke.""")
    titleColor = T.Unicode(allow_none=True, default_value=None, help="""Optional mark property definitions for custom legend styling.""")
    titleFont = T.Unicode(allow_none=True, default_value=None, help="""The font of the legend title.""")
    titleFontSize = T.CFloat(allow_none=True, default_value=None, help="""The font size of the legend title.""")
    titleFontWeight = T.Unicode(allow_none=True, default_value=None, help="""The font weight of the legend title.""")
    
    def __init__(self, gradientHeight=None, gradientStrokeColor=None, gradientStrokeWidth=None, gradientWidth=None, labelAlign=None, labelBaseline=None, labelColor=None, labelFont=None, labelFontSize=None, labelOffset=None, margin=None, offset=None, orient=None, padding=None, properties=None, shortTimeLabels=None, symbolColor=None, symbolShape=None, symbolSize=None, symbolStrokeWidth=None, titleColor=None, titleFont=None, titleFontSize=None, titleFontWeight=None, **kwargs):
        kwds = dict(gradientHeight=gradientHeight, gradientStrokeColor=gradientStrokeColor, gradientStrokeWidth=gradientStrokeWidth, gradientWidth=gradientWidth, labelAlign=labelAlign, labelBaseline=labelBaseline, labelColor=labelColor, labelFont=labelFont, labelFontSize=labelFontSize, labelOffset=labelOffset, margin=margin, offset=offset, orient=orient, padding=padding, properties=properties, shortTimeLabels=shortTimeLabels, symbolColor=symbolColor, symbolShape=symbolShape, symbolSize=symbolSize, symbolStrokeWidth=symbolStrokeWidth, titleColor=titleColor, titleFont=titleFont, titleFontSize=titleFontSize, titleFontWeight=titleFontWeight)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(LegendConfig, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .axisorient import AxisOrient


class Axis(BaseObject):
    """Wrapper for Vega-Lite Axis definition.
    
    Attributes
    ----------
    axisColor: Unicode
        Color of axis line.
    axisWidth: CFloat
        Width of the axis line.
    characterWidth: CFloat
        Character width for automatically determining title max length.
    format: Unicode
        The formatting pattern for axis labels.
    grid: Bool
        A flag indicate if gridlines should be created in addition to ticks.
    gridColor: Unicode
        Color of gridlines.
    gridDash: List(CFloat)
        The offset (in pixels) into which to begin drawing with the grid dash array.
    gridOpacity: CFloat
        The stroke opacity of grid (value between [0,1]).
    gridWidth: CFloat
        The grid width, in pixels.
    labelAlign: Unicode
        Text alignment for the Label.
    labelAngle: CFloat
        The rotation angle of the axis labels.
    labelBaseline: Unicode
        Text baseline for the label.
    labelMaxLength: CFloat
        Truncate labels that are too long.
    labels: Bool
        Enable or disable labels.
    layer: Unicode
        A string indicating if the axis (and any gridlines) should be placed above or below the data marks.
    offset: CFloat
        The offset, in pixels, by which to displace the axis from the edge of the enclosing group or data rectangle.
    orient: AxisOrient
        The orientation of the axis.
    properties: Any
        Optional mark property definitions for custom axis styling.
    shortTimeLabels: Bool
        Whether month and day names should be abbreviated.
    subdivide: CFloat
        If provided, sets the number of minor ticks between major ticks (the value 9 results in decimal subdivision).
    tickColor: Unicode
        The color of the axis's tick.
    tickLabelColor: Unicode
        The color of the tick label, can be in hex color code or regular color name.
    tickLabelFont: Unicode
        The font of the tick label.
    tickLabelFontSize: CFloat
        The font size of label, in pixels.
    tickPadding: CFloat
        The padding, in pixels, between ticks and text labels.
    tickSize: CFloat
        The size, in pixels, of major, minor and end ticks.
    tickSizeEnd: CFloat
        The size, in pixels, of end ticks.
    tickSizeMajor: CFloat
        The size, in pixels, of major ticks.
    tickSizeMinor: CFloat
        The size, in pixels, of minor ticks.
    tickWidth: CFloat
        The width, in pixels, of ticks.
    ticks: CFloat
        A desired number of ticks, for axes visualizing quantitative scales.
    title: Unicode
        A title for the axis.
    titleColor: Unicode
        Color of the title, can be in hex color code or regular color name.
    titleFont: Unicode
        Font of the title.
    titleFontSize: CFloat
        Size of the title.
    titleFontWeight: Unicode
        Weight of the title.
    titleMaxLength: CFloat
        Max length for axis title if the title is automatically generated from the field's description.
    titleOffset: CFloat
        A title offset value for the axis.
    values: List(CFloat)
        
    """
    axisColor = T.Unicode(allow_none=True, default_value=None, help="""Color of axis line.""")
    axisWidth = T.CFloat(allow_none=True, default_value=None, help="""Width of the axis line.""")
    characterWidth = T.CFloat(allow_none=True, default_value=None, help="""Character width for automatically determining title max length.""")
    format = T.Unicode(allow_none=True, default_value=None, help="""The formatting pattern for axis labels.""")
    grid = T.Bool(allow_none=True, default_value=None, help="""A flag indicate if gridlines should be created in addition to ticks.""")
    gridColor = T.Unicode(allow_none=True, default_value=None, help="""Color of gridlines.""")
    gridDash = T.List(T.CFloat(), allow_none=True, default_value=None, help="""The offset (in pixels) into which to begin drawing with the grid dash array.""")
    gridOpacity = T.CFloat(allow_none=True, default_value=None, help="""The stroke opacity of grid (value between [0,1]).""")
    gridWidth = T.CFloat(allow_none=True, default_value=None, help="""The grid width, in pixels.""")
    labelAlign = T.Unicode(allow_none=True, default_value=None, help="""Text alignment for the Label.""")
    labelAngle = T.CFloat(allow_none=True, default_value=None, help="""The rotation angle of the axis labels.""")
    labelBaseline = T.Unicode(allow_none=True, default_value=None, help="""Text baseline for the label.""")
    labelMaxLength = T.CFloat(allow_none=True, default_value=None, min=1, help="""Truncate labels that are too long.""")
    labels = T.Bool(allow_none=True, default_value=None, help="""Enable or disable labels.""")
    layer = T.Unicode(allow_none=True, default_value=None, help="""A string indicating if the axis (and any gridlines) should be placed above or below the data marks.""")
    offset = T.CFloat(allow_none=True, default_value=None, help="""The offset, in pixels, by which to displace the axis from the edge of the enclosing group or data rectangle.""")
    orient = AxisOrient(allow_none=True, default_value=None, help="""The orientation of the axis.""")
    properties = T.Any(allow_none=True, default_value=None, help="""Optional mark property definitions for custom axis styling.""")
    shortTimeLabels = T.Bool(allow_none=True, default_value=None, help="""Whether month and day names should be abbreviated.""")
    subdivide = T.CFloat(allow_none=True, default_value=None, help="""If provided, sets the number of minor ticks between major ticks (the value 9 results in decimal subdivision).""")
    tickColor = T.Unicode(allow_none=True, default_value=None, help="""The color of the axis's tick.""")
    tickLabelColor = T.Unicode(allow_none=True, default_value=None, help="""The color of the tick label, can be in hex color code or regular color name.""")
    tickLabelFont = T.Unicode(allow_none=True, default_value=None, help="""The font of the tick label.""")
    tickLabelFontSize = T.CFloat(allow_none=True, default_value=None, help="""The font size of label, in pixels.""")
    tickPadding = T.CFloat(allow_none=True, default_value=None, help="""The padding, in pixels, between ticks and text labels.""")
    tickSize = T.CFloat(allow_none=True, default_value=None, min=0, help="""The size, in pixels, of major, minor and end ticks.""")
    tickSizeEnd = T.CFloat(allow_none=True, default_value=None, min=0, help="""The size, in pixels, of end ticks.""")
    tickSizeMajor = T.CFloat(allow_none=True, default_value=None, min=0, help="""The size, in pixels, of major ticks.""")
    tickSizeMinor = T.CFloat(allow_none=True, default_value=None, min=0, help="""The size, in pixels, of minor ticks.""")
    tickWidth = T.CFloat(allow_none=True, default_value=None, help="""The width, in pixels, of ticks.""")
    ticks = T.CFloat(allow_none=True, default_value=None, min=0, help="""A desired number of ticks, for axes visualizing quantitative scales.""")
    title = T.Unicode(allow_none=True, default_value=None, help="""A title for the axis.""")
    titleColor = T.Unicode(allow_none=True, default_value=None, help="""Color of the title, can be in hex color code or regular color name.""")
    titleFont = T.Unicode(allow_none=True, default_value=None, help="""Font of the title.""")
    titleFontSize = T.CFloat(allow_none=True, default_value=None, help="""Size of the title.""")
    titleFontWeight = T.Unicode(allow_none=True, default_value=None, help="""Weight of the title.""")
    titleMaxLength = T.CFloat(allow_none=True, default_value=None, min=0, help="""Max length for axis title if the title is automatically generated from the field's description.""")
    titleOffset = T.CFloat(allow_none=True, default_value=None, help="""A title offset value for the axis.""")
    values = T.List(T.CFloat(), allow_none=True, default_value=None)
    
    def __init__(self, axisColor=None, axisWidth=None, characterWidth=None, format=None, grid=None, gridColor=None, gridDash=None, gridOpacity=None, gridWidth=None, labelAlign=None, labelAngle=None, labelBaseline=None, labelMaxLength=None, labels=None, layer=None, offset=None, orient=None, properties=None, shortTimeLabels=None, subdivide=None, tickColor=None, tickLabelColor=None, tickLabelFont=None, tickLabelFontSize=None, tickPadding=None, tickSize=None, tickSizeEnd=None, tickSizeMajor=None, tickSizeMinor=None, tickWidth=None, ticks=None, title=None, titleColor=None, titleFont=None, titleFontSize=None, titleFontWeight=None, titleMaxLength=None, titleOffset=None, values=None, **kwargs):
        kwds = dict(axisColor=axisColor, axisWidth=axisWidth, characterWidth=characterWidth, format=format, grid=grid, gridColor=gridColor, gridDash=gridDash, gridOpacity=gridOpacity, gridWidth=gridWidth, labelAlign=labelAlign, labelAngle=labelAngle, labelBaseline=labelBaseline, labelMaxLength=labelMaxLength, labels=labels, layer=layer, offset=offset, orient=orient, properties=properties, shortTimeLabels=shortTimeLabels, subdivide=subdivide, tickColor=tickColor, tickLabelColor=tickLabelColor, tickLabelFont=tickLabelFont, tickLabelFontSize=tickLabelFontSize, tickPadding=tickPadding, tickSize=tickSize, tickSizeEnd=tickSizeEnd, tickSizeMajor=tickSizeMajor, tickSizeMinor=tickSizeMinor, tickWidth=tickWidth, ticks=ticks, title=title, titleColor=titleColor, titleFont=titleFont, titleFontSize=titleFontSize, titleFontWeight=titleFontWeight, titleMaxLength=titleMaxLength, titleOffset=titleOffset, values=values)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Axis, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .nicetime import NiceTime
from .scaletype import ScaleType


class Scale(BaseObject):
    """Wrapper for Vega-Lite Scale definition.
    
    Attributes
    ----------
    bandSize: CFloat
        
    clamp: Bool
        If true, values that exceed the data domain are clamped to either the minimum or maximum range value.
    domain: Union(Unicode, List(CFloat), List(Unicode))
        The domain of the scale, representing the set of data values.
    exponent: CFloat
        Sets the exponent of the scale transformation.
    nice: Union(Bool, NiceTime)
        If specified, modifies the scale domain to use a more human-friendly value range.
    padding: CFloat
        Applies spacing among ordinal elements in the scale range.
    range: Union(Unicode, List(CFloat), List(Unicode))
        The range of the scale, representing the set of visual values.
    round: Bool
        If true, rounds numeric output values to integers.
    type: ScaleType
        
    useRawDomain: Bool
        Uses the source data range as scale domain instead of aggregated data for aggregate axis.
    zero: Bool
        If true, ensures that a zero baseline value is included in the scale domain.
    """
    bandSize = T.CFloat(allow_none=True, default_value=None, min=0)
    clamp = T.Bool(allow_none=True, default_value=None, help="""If true, values that exceed the data domain are clamped to either the minimum or maximum range value.""")
    domain = T.Union([T.Unicode(allow_none=True, default_value=None), T.List(T.CFloat(), allow_none=True, default_value=None), T.List(T.Unicode(), allow_none=True, default_value=None)])
    exponent = T.CFloat(allow_none=True, default_value=None, help="""Sets the exponent of the scale transformation.""")
    nice = T.Union([T.Bool(allow_none=True, default_value=None), NiceTime(allow_none=True, default_value=None)])
    padding = T.CFloat(allow_none=True, default_value=None, help="""Applies spacing among ordinal elements in the scale range.""")
    range = T.Union([T.Unicode(allow_none=True, default_value=None), T.List(T.CFloat(), allow_none=True, default_value=None), T.List(T.Unicode(), allow_none=True, default_value=None)])
    round = T.Bool(allow_none=True, default_value=None, help="""If true, rounds numeric output values to integers.""")
    type = ScaleType(allow_none=True, default_value=None)
    useRawDomain = T.Bool(allow_none=True, default_value=None, help="""Uses the source data range as scale domain instead of aggregated data for aggregate axis.""")
    zero = T.Bool(allow_none=True, default_value=None, help="""If true, ensures that a zero baseline value is included in the scale domain.""")
    
    def __init__(self, bandSize=None, clamp=None, domain=None, exponent=None, nice=None, padding=None, range=None, round=None, type=None, useRawDomain=None, zero=None, **kwargs):
        kwds = dict(bandSize=bandSize, clamp=clamp, domain=domain, exponent=exponent, nice=nice, padding=padding, range=range, round=round, type=type, useRawDomain=useRawDomain, zero=zero)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Scale, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

from .channel_wrappers import ChannelWithLegend, Field, OrderChannel, PositionChannel


class Color(ChannelWithLegend):
    pass


class Column(PositionChannel):
    pass


class Detail(Field):
    pass


class Label(Field):
    pass


class Opacity(ChannelWithLegend):
    pass


class Order(OrderChannel):
    pass


class Path(OrderChannel):
    pass


class Row(PositionChannel):
    pass


class Shape(ChannelWithLegend):
    pass


class Size(ChannelWithLegend):
    pass


class Text(Field):
    pass


class X(PositionChannel):
    pass


class Y(PositionChannel):
    pass





import os
import json

SCHEMA_FILE = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                           'vega-lite-schema.json')

def load_schema():
    """Load the VegaLite Schema as a Python dictionary"""
    with open(SCHEMA_FILE) as f:
        return json.load(f)



import os
import json
import pandas as pd

try:
    from urllib.error import URLError, HTTPError
    from urllib.request import urlopen
except ImportError:
    # Python 2.X
    from urllib2 import URLError, HTTPError, urlopen

try:
    from functools import lru_cache
except ImportError:
    # Python 2.X: function not available
    lru_cache = lambda maxsize=128, typed=False: (lambda y: y)


BASE_URL = 'https://vega.github.io/vega-datasets/data/'


@lru_cache()
def _datasets_json():
    json_file = os.path.join(os.path.dirname(__file__), 'datasets.json')
    with open(json_file) as f:
        return json.loads(f.read())


def connection_ok():
    """Check web connection.

    Returns True if web connection is OK, False otherwise.
    """
    try:
        response = urlopen(BASE_URL, timeout=1)
        # if an index page is ever added, this will pass through
        return True
    except HTTPError:
        # There's no index for BASE_URL so Error 404 is expected
        return True
    except URLError:
        # This is raised if there is no internet connection
        return False


def list_datasets():
    """List the available datasets."""
    return list(_datasets_json().keys())


@lru_cache()
def load_dataset(name):
    """Load a dataset by name as a pandas.DataFrame."""
    item = _datasets_json().get(name, None)
    if item is None:
        raise ValueError('No such dataset {0} exists, '
                         'use list_datasets to get a list'.format(name))
    url = BASE_URL + item['filename']
    if item['format'] == 'json':
        return pd.read_json(url)
    elif item['format'] == 'csv':
        return pd.read_csv(url)



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class TimeUnit(T.Enum):
    """One of ['year', 'month', 'day', 'date', 'hours', 'minutes', 'seconds', 'milliseconds', 'yearmonth', 'yearmonthday', 'yearmonthdate', 'yearday', 'yeardate', 'yearmonthdayhours', 'yearmonthdayhoursminutes', 'yearmonthdayhoursminutesseconds', 'hoursminutes', 'hoursminutesseconds', 'minutesseconds', 'secondsmilliseconds']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(TimeUnit, self).__init__(['year', 'month', 'day', 'date', 'hours', 'minutes', 'seconds', 'milliseconds', 'yearmonth', 'yearmonthday', 'yearmonthdate', 'yearday', 'yeardate', 'yearmonthdayhours', 'yearmonthdayhoursminutes', 'yearmonthdayhoursminutesseconds', 'hoursminutes', 'hoursminutesseconds', 'minutesseconds', 'secondsmilliseconds'],
                                    default_value=default_value,
                                    **metadata)


import pandas as pd
import traitlets as T

from ..utils._py3k_compat import string_types

_attr_template = "Attribute not found: {0}. Valid keyword arguments for this class: {1}"


class BaseObject(T.HasTraits):

    skip = []

    def __init__(self, **kwargs):
        all_traits = list(self.traits())
        for k in kwargs:
            if k not in all_traits:
                raise KeyError(_attr_template.format(k, all_traits))
        super(BaseObject, self).__init__(**kwargs)

    @classmethod
    def infer_keywords(cls, *args, **kwargs):
        """Utility to initialize object from args and kwargs

        Arguments are converted to keyword arguments by inferring the keyword
        from their type.
        Keyword arguments are converted to the correct Instance class
        if required.
        """
        def get_class(trait):
            # TODO: what do do with lists?
            if isinstance(trait, T.Union):
                for klass in map(get_class, trait.trait_types):
                    if klass:
                        return klass
            elif isinstance(trait, T.Instance):
                return trait.klass

        traits = cls.class_traits()
        classes = {n: get_class(t) for n, t in traits.items()}

        # Turn all keyword arguments to the appropriate class
        for name, arg in kwargs.items():
            Trait = classes.get(name, None)
            if Trait is not None and not isinstance(arg, Trait):
                try:
                    kwargs[name] = Trait(arg)
                except (TypeError, T.TraitError):
                    pass  # errors will handled by traitlets below

        # find forward/backward mapping among unique classes
        name_to_trait = {}
        while classes:
            name, trait = classes.popitem()
            if trait is None:
                continue
            if trait not in set.union(set(classes.values()),
                                      set(name_to_trait.values())):
                name_to_trait[name] = trait
        trait_to_name = {t: n for n, t in name_to_trait.items()}

        # Update all arguments
        for arg in args:
            name = trait_to_name.get(type(arg), None)
            if name is None:
                raise ValueError("{0}: Unable to infer argument name for {1}".format(cls, arg))
            elif name in kwargs:
                raise ValueError("{0}: {1} specified both by arg and kwarg".format(cls, name))
            else:
                kwargs[name] = arg
        return kwargs

    def update_traits(self, **kwargs):
        for key, val in kwargs.items():
            self.set_trait(key, val)
        return self

    def update_inferred_traits(self, *args, **kwargs):
        kwargs = self.infer_keywords(*args, **kwargs)
        return self.update_traits(**kwargs)

    def update_subtraits(self, attrs, *args, **kwargs):
        """Update sub-traits without overwriting other traits"""
        if not (args or kwargs):
            return self
        if isinstance(attrs, string_types):
            attrs = (attrs,)
        if len(attrs) == 0:
            self.update_inferred_traits(*args, **kwargs)
        else:
            attr = attrs[0]
            if attr not in self.traits():
                raise ValueError('{0} has no trait {1}'.format(self, attr))
            trait = getattr(self, attr)
            if trait is None:
                trait = self.traits()[attr].klass()
            setattr(self, attr, trait.update_subtraits(attrs[1:], *args, **kwargs))
        return self

    def __contains__(self, key):
        try:
            value = getattr(self, key)
        except AttributeError:
            return False

        # comparison to None will break, so check DataFrame specifically
        if isinstance(value, pd.DataFrame):
            return True
        elif value is not None:
            if isinstance(value, (int, float, bool)):
                return True
            else:
                return bool(value)
        else:
            return False

    def __dir__(self):
        """Customize tab completed attributes."""
        return list(self.traits())

    @classmethod
    def from_dict(cls, dct):
        """Instantiate the object from a valid JSON dictionary"""
        from ..utils.visitors import FromDict
        return FromDict().clsvisit(cls, dct)

    def to_dict(self, data=True):
        """Emit the JSON representation for this object as as dict."""
        from ..utils.visitors import ToDict
        self._finalize()
        return ToDict().visit(self, data)

    def _finalize(self, **kwargs):
        """Finalize the object, and all contained objects, for export."""
        def finalize_obj(obj):
            if isinstance(obj, BaseObject):
                obj._finalize(**kwargs)
            elif isinstance(obj, list):
                for item in obj:
                    finalize_obj(item)

        for name in self.traits():
            value = getattr(self, name)
            finalize_obj(value)



__version__ = '1.0.0rc2'

from .api import (
    load_vegalite_spec,
    Label,
    Formula,
    StackOffset,
    Data,
    FacetScaleConfig,
    AxisConfig,
    Shape,
    Axis,
    AggregateOp,
    ScaleConfig,
    NiceTime,
    Transform,
    VerticalAlign,
    SortOrder,
    Legend,
    LegendConfig,
    Column,
    FontWeight,
    SortField,
    Text,
    MarkConfig,
    TimeUnit,
    FacetConfig,
    FontStyle,
    Config,
    Order,
    HorizontalAlign,
    Path,
    Scale,
    Encoding,
    Facet,
    Size,
    FacetGridConfig,
    Row,
    Bin,
    AxisOrient,
    X, Y,
    Color,
    Opacity,
    DataFormat,
    Chart,
    LayeredChart,
    FacetedChart,
    CellConfig,
    Detail,
)

from .datasets import (
    list_datasets,
    load_dataset
)

from .tutorial import tutorial




import warnings

import pandas as pd
import traitlets as T

from ._py3k_compat import integer_types, string_types
from .. import schema
from .codegen import CodeGen
from . import sanitize_dataframe, construct_shorthand


class Visitor(object):
    """Class implementing the external visitor pattern"""
    def visit(self, obj, *args, **kwargs):
        for cls in obj.__class__.__mro__:
            method = getattr(self, 'visit_' + cls.__name__, None)
            if method:
                break
        else:
            method = self.generic_visit
        return method(obj, *args, **kwargs)

    def clsvisit(self, obj, *args, **kwargs):
        for cls in obj.__mro__:
            method = getattr(self, 'clsvisit_' + cls.__name__, None)
            if method:
                break
        else:
            method = self.generic_clsvisit
        try:
            return method(obj, *args, **kwargs)
        except KeyError:
            print(obj)
            print(args)
            print(kwargs)
            raise


class ToDict(Visitor):
    """Crawl object structure to output dictionary"""
    def generic_visit(self, obj, *args, **kwargs):
        return obj

    def visit_list(self, obj, *args, **kwargs):
            return [self.visit(o) for o in obj]

    def visit_BaseObject(self, obj, *args, **kwargs):
        D = {}
        for k in obj.traits():
            if k in obj and k not in obj.skip:
                v = getattr(obj, k)
                if v is not None:
                    D[k] = self.visit(v)
        return D

    def _visit_with_data(self, obj, data=True):
        D = self.visit_BaseObject(obj)
        if data:
            if isinstance(obj.data, schema.Data):
                D['data'] = self.visit(obj.data)
            elif isinstance(obj.data, pd.DataFrame):
                values = sanitize_dataframe(obj.data).to_dict(orient='records')
                D['data'] = self.visit(schema.Data(values=values))
        else:
            D.pop('data', None)
        return D

    visit_Chart = _visit_with_data
    visit_LayeredChart = _visit_with_data
    visit_FacetedChart = _visit_with_data


class ToCode(Visitor):
    """Crawl object structure to output code"""
    def generic_visit(self, obj, *args, **kwargs):
        return repr(obj)

    def visit_list(self, obj, *args, **kwargs):
        return [self.visit(o) for o in obj]

    def visit_BaseObject(self, obj, *args, **kwargs):
        kwds = {k: getattr(obj, k) for k in obj.traits()
                if k not in obj.skip and k in obj}
        kwds = {k: self.visit(v) for k, v in kwds.items()}
        return CodeGen(obj.__class__.__name__, kwargs=kwds)

    def _visit_ChannelWrapper(self, obj, *args, **kwargs):
        if obj.shorthand:
            shorthand = obj.shorthand
        else:
            shorthand = construct_shorthand(field=obj.field,
                                            aggregate=obj.aggregate,
                                            type=obj.type)
        code = self.visit_BaseObject(obj)
        if shorthand:
            code.add_args(repr(shorthand))
        code.remove_kwargs('field', 'aggregate', 'type')

        do_shorten = (shorthand
                      and kwargs.get('shorten', False)
                      and len(code.args) == 1
                      and not (code.kwargs or code.methods))
        if do_shorten:
            return repr(shorthand)
        else:
            return code

    def visit_ChannelWithLegend(self, obj, *args, **kwargs):
        return self._visit_ChannelWrapper(obj, *args, **kwargs)

    def visit_Field(self, obj, *args, **kwargs):
        return self._visit_ChannelWrapper(obj, *args, **kwargs)

    def visit_OrderChannel(self, obj, *args, **kwargs):
        return self._visit_ChannelWrapper(obj, *args, **kwargs)

    def visit_PositionChannel(self, obj, *args, **kwargs):
        return self._visit_ChannelWrapper(obj, *args, **kwargs)

    def visit_Encoding(self, obj, *args, **kwargs):
        kwds = {k: getattr(obj, k) for k in obj.traits()
                if k not in obj.skip and k in obj}
        kwds = {k: self.visit(v, shorten=True) for k, v in kwds.items()}
        return CodeGen(obj.__class__.__name__, kwargs=kwds)

    def _visit_with_data(self, obj, data=None, *args, **kwargs):
        code = self.visit_BaseObject(obj)

        # Add data as a top-level argument
        if data is not None:
            # data as a user-speficied name
            code.add_args(data)
        elif isinstance(obj.data, schema.Data):
            # data as a Data trait: return raw URL if possible
            url_only = ('url' in obj.data and
                        'values' not in obj.data and
                        'formatType' not in obj.data)
            if url_only:
                code.add_args("'{0}'".format(obj.data.url))
            else:
                code.add_args(self.visit(obj.data))
        elif isinstance(obj.data, pd.DataFrame):
            # data as a dataframe; this gets too long so we leave it out
            warnings.warn("Skipping dataframe definition in altair code")

        return code

    def visit_Chart(self, obj, data=None, *args, **kwargs):
        code = self._visit_with_data(obj, data, *args, **kwargs)

        # enable Chart().mark_point(**kwargs)
        mark = code.kwargs.pop('mark', None)
        if mark:
            config = code.kwargs.get('config', CodeGen(''))
            markconfig = config.kwargs.pop('mark', CodeGen(''))
            code.add_methods(markconfig.rename('mark_{0}'.format(obj.mark)))

        # enable Chart().encode(**kwargs)
        code.convert_arg_to_method('encoding', 'encode', depth=1)

        # enable Chart().transform_data(**kwargs)
        code.convert_arg_to_method('transform', 'transform_data', depth=1)

        # enable Chart().configure(**kwargs)
        #                configure_<arg>(**kwargs)
        #                configure_facet_<arg>(**kwargs)
        code.convert_arg_to_method('config', 'configure', depth=3)

        return code

    def visit_LayeredChart(self, obj, data=None, *args, **kwargs):
        return self._visit_with_data(obj, data, *args, **kwargs)

    def visit_FacetedChart(self, obj, data=None, *args, **kwargs):
        return self._visit_with_data(obj, data, *args, **kwargs)


class FromDict(Visitor):
    """Crawl object structure to construct object from a Dictionary"""
    def clsvisit_BaseObject(self, cls, dct, *args, **kwargs):
        try:
            obj = cls()
        except TypeError as err:  # Argument missing
            obj = cls('')

        for prop, val in dct.items():
            subtrait = obj.traits()[prop]
            obj.set_trait(prop, self.visit(subtrait, val))
        return obj

    def _clsvisit_with_data(self, cls, dct, *args, **kwargs):
        # Remove data first and handle it specially later
        if 'data' in dct:
            dct = dct.copy()
        data = dct.pop('data', None)

        obj = self.clsvisit_BaseObject(cls, dct)

        # data is not a typical trait; do special handling here.
        if data is not None:
            values_only = ('values' in data and
                           'url' not in data and
                           'dataFormat' not in data)
            if values_only:
                obj.data = pd.DataFrame(data['values'])
            else:
                obj.data = schema.Data(**data)
        return obj

    clsvisit_Chart = _clsvisit_with_data
    clsvisit_LayeredChart = _clsvisit_with_data
    clsvisit_FacetedChart = _clsvisit_with_data

    def visit_List(self, trait, dct, *args, **kwargs):
        return [self.visit(trait._trait, item) for item in dct]

    def visit_Instance(self, trait, dct, *args, **kwargs):
        try:
            return self.generic_visit(trait, dct)
        except T.TraitError:
            return self.clsvisit(trait.klass, dct)

    def visit_Union(self, trait, dct, *args, **kwargs):
        try:
            return self.generic_visit(trait, dct)
        except T.TraitError:
            for subtrait in trait.trait_types:
                try:
                    return self.visit(subtrait, dct)
                except T.TraitError:
                    pass
            raise  # no valid trait found

    def generic_visit(self, trait, dct, *args, **kwargs):
        # pass-through simple types
        if isinstance(dct, (integer_types, string_types, bool, float)):
            return dct
        else:
            raise T.TraitError('cannot set {0} to {1}'.format(trait, dct))



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class NiceTime(T.Enum):
    """One of ['second', 'minute', 'hour', 'day', 'week', 'month', 'year']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(NiceTime, self).__init__(['second', 'minute', 'hour', 'day', 'week', 'month', 'year'],
                                    default_value=default_value,
                                    **metadata)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .axisconfig import AxisConfig
from .cellconfig import CellConfig
from .facetgridconfig import FacetGridConfig
from .facetscaleconfig import FacetScaleConfig


class FacetConfig(BaseObject):
    """Wrapper for Vega-Lite FacetConfig definition.
    
    Attributes
    ----------
    axis: AxisConfig
        Facet Axis Config.
    cell: CellConfig
        Facet Cell Config.
    grid: FacetGridConfig
        Facet Grid Config.
    scale: FacetScaleConfig
        Facet Scale Config.
    """
    axis = T.Instance(AxisConfig, allow_none=True, default_value=None, help="""Facet Axis Config.""")
    cell = T.Instance(CellConfig, allow_none=True, default_value=None, help="""Facet Cell Config.""")
    grid = T.Instance(FacetGridConfig, allow_none=True, default_value=None, help="""Facet Grid Config.""")
    scale = T.Instance(FacetScaleConfig, allow_none=True, default_value=None, help="""Facet Scale Config.""")
    
    def __init__(self, axis=None, cell=None, grid=None, scale=None, **kwargs):
        kwds = dict(axis=axis, cell=cell, grid=grid, scale=scale)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(FacetConfig, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class Mark(T.Enum):
    """One of ['area', 'bar', 'line', 'point', 'text', 'tick', 'rule', 'circle', 'square']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(Mark, self).__init__(['area', 'bar', 'line', 'point', 'text', 'tick', 'rule', 'circle', 'square'],
                                    default_value=default_value,
                                    **metadata)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .positionchanneldef import PositionChannelDef


class Facet(BaseObject):
    """Wrapper for Vega-Lite Facet definition.
    
    Attributes
    ----------
    column: PositionChannelDef
        
    row: PositionChannelDef
        
    """
    column = T.Instance(PositionChannelDef, allow_none=True, default_value=None)
    row = T.Instance(PositionChannelDef, allow_none=True, default_value=None)
    
    def __init__(self, column=None, row=None, **kwargs):
        kwds = dict(column=column, row=row)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Facet, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject


class FacetScaleConfig(BaseObject):
    """Wrapper for Vega-Lite FacetScaleConfig definition.
    
    Attributes
    ----------
    padding: CFloat
        
    round: Bool
        
    """
    padding = T.CFloat(allow_none=True, default_value=None)
    round = T.Bool(allow_none=True, default_value=None)
    
    def __init__(self, padding=None, round=None, **kwargs):
        kwds = dict(padding=padding, round=round)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(FacetScaleConfig, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T

from .. import _interface as schema

from .named_channels import Color, Column, Detail, Label, Opacity, Order, Path, Row, Shape, Size, Text, X, Y


class Encoding(schema.Encoding):
    """Object for storing channel encodings

    Attributes
    ----------
    color: Color
        Color of the marks - either fill or stroke color based on mark type.
    column: Column
        Horizontal facets for trellis plots.
    detail: Union(Detail, List(Detail))
        Additional levels of detail for grouping data in aggregate views and in line and area marks without mapping data to a specific visual channel.
    label: Label
        
    opacity: Opacity
        Opacity of the marks - either can be a value or in a range.
    order: Union(Order, List(Order))
        Layer order for non-stacked marks, or stack order for stacked marks.
    path: Union(Path, List(Path))
        Order of data points in line marks.
    row: Row
        Vertical facets for trellis plots.
    shape: Shape
        The symbol's shape (only for `point` marks).
    size: Size
        Size of the mark.
    text: Text
        Text of the `text` mark.
    x: X
        X coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).
    y: Y
        Y coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).
    """
    color = T.Instance(Color, allow_none=True, default_value=None, help="""Color of the marks - either fill or stroke color based on mark type.""")
    column = T.Instance(Column, allow_none=True, default_value=None, help="""Horizontal facets for trellis plots.""")
    detail = T.Union([T.Instance(Detail, allow_none=True, default_value=None, help="""Interface for any kind of FieldDef."""), T.List(T.Instance(Detail, help="""Interface for any kind of FieldDef."""), allow_none=True, default_value=None)])
    label = T.Instance(Label, allow_none=True, default_value=None)
    opacity = T.Instance(Opacity, allow_none=True, default_value=None, help="""Opacity of the marks - either can be a value or in a range.""")
    order = T.Union([T.Instance(Order, allow_none=True, default_value=None), T.List(T.Instance(Order), allow_none=True, default_value=None)])
    path = T.Union([T.Instance(Path, allow_none=True, default_value=None), T.List(T.Instance(Path), allow_none=True, default_value=None)])
    row = T.Instance(Row, allow_none=True, default_value=None, help="""Vertical facets for trellis plots.""")
    shape = T.Instance(Shape, allow_none=True, default_value=None, help="""The symbol's shape (only for `point` marks).""")
    size = T.Instance(Size, allow_none=True, default_value=None, help="""Size of the mark.""")
    text = T.Instance(Text, allow_none=True, default_value=None, help="""Text of the `text` mark.""")
    x = T.Instance(X, allow_none=True, default_value=None, help="""X coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).""")
    y = T.Instance(Y, allow_none=True, default_value=None, help="""Y coordinates for `point`, `circle`, `square`, `line`, `rule`, `text`, and `tick` (or to width and height for `bar` and `area` marks).""")
    
    channel_names = ['color', 'column', 'detail', 'label', 'opacity', 'order', 'path', 'row', 'shape', 'size', 'text', 'x', 'y']
    skip = ['channel_names']


class Facet(schema.Facet):
    """Object for storing channel encodings

    Attributes
    ----------
    column: Column
        
    row: Row
        
    """
    column = T.Instance(Column, allow_none=True, default_value=None)
    row = T.Instance(Row, allow_none=True, default_value=None)
    
    channel_names = ['column', 'row']
    skip = ['channel_names']





# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from .unitspec import UnitSpec
from .config import Config
from .data import Data
from .encoding import Encoding
from .mark import Mark
from .transform import Transform


class ExtendedUnitSpec(UnitSpec):
    """Wrapper for Vega-Lite ExtendedUnitSpec definition.
    
    Attributes
    ----------
    config: Config
        Configuration object.
    data: Data
        An object describing the data source.
    description: Unicode
        An optional description of this mark for commenting purpose.
    encoding: Encoding
        A key-value mapping between encoding channels and definition of fields.
    mark: Mark
        The mark type.
    name: Unicode
        Name of the visualization for later reference.
    transform: Transform
        An object describing filter and new field calculation.
    """
    config = T.Instance(Config, allow_none=True, default_value=None, help="""Configuration object.""")
    data = T.Instance(Data, allow_none=True, default_value=None, help="""An object describing the data source.""")
    description = T.Unicode(allow_none=True, default_value=None, help="""An optional description of this mark for commenting purpose.""")
    encoding = T.Instance(Encoding, allow_none=True, default_value=None, help="""A key-value mapping between encoding channels and definition of fields.""")
    mark = Mark(allow_none=True, default_value=None, help="""The mark type.""")
    name = T.Unicode(allow_none=True, default_value=None, help="""Name of the visualization for later reference.""")
    transform = T.Instance(Transform, allow_none=True, default_value=None, help="""An object describing filter and new field calculation.""")
    
    def __init__(self, config=None, data=None, description=None, encoding=None, mark=None, name=None, transform=None, **kwargs):
        kwds = dict(config=config, data=data, description=description, encoding=encoding, mark=mark, name=name, transform=transform)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(ExtendedUnitSpec, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class FontWeight(T.Enum):
    """One of ['normal', 'bold']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(FontWeight, self).__init__(['normal', 'bold'],
                                    default_value=default_value,
                                    **metadata)


LONG_DESCRIPTION = """
This package provides a Python API for building statistical visualizations in a
declarative manner. This API contains no actual visualization rendering code, but
instead emits JSON data structures following the `Vega-Lite`_ specification. For
convenience, Altair can optionally use `ipyvega`_ to seamlessly display client-side
renderings in the Jupyter notebook.

Visit our GitHub repository (https://github.com/ellisonbg/altair) for
installation instructions, examples and documentation.

.. image:: https://raw.githubusercontent.com/ellisonbg/altair/master/images/cars.png

.. _Vega-Lite: https://github.com/vega/vega-lite
.. _ipyvega: https://github.com/vega/ipyvega
"""

DESCRIPTION         = "Altair: A high-level declarative visualization library for Python."
NAME                = "altair"
PACKAGES            = ['altair',
                       'altair.tests',
                       'altair.datasets',
                       'altair.examples',
                       'altair.examples.tests',
                       'altair.utils',
                       'altair.utils.tests',
                       'altair.schema',
                       'altair.schema._wrappers',
                       'altair.schema._interface',
                       'altair.schema._interface.tests']
PACKAGE_DATA        = {'altair': ['schema/*.json',
                                  'examples/json/*.json',
                                  'datasets/*.json']}
AUTHOR              = "Brian E. Granger / Jake VanderPlas"
AUTHOR_EMAIL        = "ellisonbg@gmail.com / jakevdp@cs.washington.edu"
URL                 = 'http://github.com/ellisonbg/altair/'
DOWNLOAD_URL        = 'http://github.com/ellisonbg/altair/'
LICENSE             = 'BSD 3-clause'
INSTALL_REQUIRES    = ['traitlets','ipython','pandas','vega>=0.4.1']


import io
import os
import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def read(path, encoding='utf-8'):
    path = os.path.join(os.path.dirname(__file__), path)
    with io.open(path, encoding=encoding) as fp:
        return fp.read()


def version(path):
    """Obtain the packge version from a python file e.g. pkg/__init__.py

    See <https://packaging.python.org/en/latest/single_source_version.html>.
    """
    version_file = read(path)
    version_match = re.search(r"""^__version__ = ['"]([^'"]*)['"]""",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


VERSION = version('altair/__init__.py')


setup(name=NAME,
      version=VERSION,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      url=URL,
      download_url=DOWNLOAD_URL,
      license=LICENSE,
      packages=PACKAGES,
      package_data=PACKAGE_DATA,
      install_requires=INSTALL_REQUIRES,
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'],
     )



"""
Utility routines
"""
import re
import warnings

import pandas as pd
import numpy as np


TYPECODE_MAP = {'ordinal': 'O',
                'nominal': 'N',
                'quantitative': 'Q',
                'temporal': 'T'}

INV_TYPECODE_MAP = {v: k for k, v in TYPECODE_MAP.items()}

TYPE_ABBR = TYPECODE_MAP.values()


def parse_shorthand(shorthand):
    """
    Parse the shorthand expression for aggregation, field, and type.

    These are of the form:

    - "col_name"
    - "col_name:O"
    - "average(col_name)"
    - "average(col_name):O"

    Parameters
    ----------
    shorthand: str
        Shorthand string

    Returns
    -------
    D : dict
        Dictionary containing the field, aggregate, and typecode
    """
    if not shorthand:
        return {}

    # Must import this here to avoid circular imports
    from ..schema import AggregateOp
    valid_aggregates = AggregateOp().values
    valid_typecodes = list(TYPECODE_MAP) + list(INV_TYPECODE_MAP)

    # build regular expressions
    units = dict(field='(?P<field>.*)',
                 type='(?P<type>{0})'.format('|'.join(valid_typecodes)),
                 aggregate='(?P<aggregate>{0})'.format('|'.join(valid_aggregates)))
    patterns = [r'{field}',
                r'{field}:{type}',
                r'{aggregate}\({field}\)',
                r'{aggregate}\({field}\):{type}']
    regexps = (re.compile('\A' + p.format(**units) + '\Z', re.DOTALL)
               for p in patterns[::-1])

    # find matches depending on valid fields passed
    match = next(exp.match(shorthand).groupdict() for exp in regexps
                 if exp.match(shorthand))

    # Use short form of the type expression
    typ = match.get('type', None)
    if typ:
        match['type'] = INV_TYPECODE_MAP.get(typ, typ)
    return match


def construct_shorthand(field=None, aggregate=None, type=None):
    """Construct a shorthand representation.

    See also: parse_shorthand"""
    if field is None:
        return ''

    sh = field

    if aggregate is not None:
        sh = '{0}({1})'.format(aggregate, sh)

    if type is not None:
        type = TYPECODE_MAP.get(type, type)
        if type not in TYPE_ABBR:
            raise ValueError('Unrecognized Type: {0}'.format(type))
        sh = '{0}:{1}'.format(sh, type)

    return sh


def infer_vegalite_type(data, field=None):
    """
    From an array-like input, infer the correct vega typecode
    ('ordinal', 'nominal', 'quantitative', or 'temporal')

    Parameters
    ----------
    data: Numpy array or Pandas Series
    field: str column name
    """
    # See if we can read the type from the field
    if field is not None:
        parsed = parse_shorthand(field)
        if parsed.get('type'):
            return parsed['type']

    # Otherwise, infer based on the dtype of the input
    typ = pd.lib.infer_dtype(data)

    # TODO: Once this returns 'O', please update test_select_x and test_select_y in test_api.py

    if typ in ['floating', 'mixed-integer-float', 'integer',
               'mixed-integer', 'complex']:
        return 'quantitative'
    elif typ in ['string', 'bytes', 'categorical', 'boolean', 'mixed', 'unicode']:
        return 'nominal'
    elif typ in ['datetime', 'datetime64', 'timedelta',
                 'timedelta64', 'date', 'time', 'period']:
        return 'temporal'
    else:
        warnings.warn("I don't know how to infer vegalite type from '{0}'.  "
                      "Defaulting to nominal.".format(typ))
        return 'nominal'


def sanitize_dataframe(df):
    """Sanitize a DataFrame to prepare it for serialization.

    * Make a copy
    * Raise ValueError if it has a hierarchical index.
    * Convert categoricals to strings.
    * Convert np.int dtypes to Python int objects
    * Convert floats to objects and replace NaNs by None.
    * Convert DateTime dtypes into appropriate string representations
    """
    df = df.copy()

    if isinstance(df.index, pd.core.index.MultiIndex):
        raise ValueError('Hierarchical indices not supported')
    if isinstance(df.columns, pd.core.index.MultiIndex):
        raise ValueError('Hierarchical indices not supported')

    for col_name, dtype in df.dtypes.iteritems():
        if str(dtype) == 'category':
            # XXXX: work around bug in to_json for categorical types
            # https://github.com/pydata/pandas/issues/10778
            df[col_name] = df[col_name].astype(str)
        elif np.issubdtype(dtype, np.integer):
            # convert integers to objects; np.int is not JSON serializable
            df[col_name] = df[col_name].astype(object)
        elif np.issubdtype(dtype, np.floating):
            # For floats, convert nan->None: np.float is not JSON serializable
            col = df[col_name].astype(object)
            df[col_name] = col.where(col.notnull(), None)
        elif str(dtype).startswith('datetime'):
            # Convert datetimes to strings
            # astype(str) will choose the appropriate resolution
            df[col_name] = df[col_name].astype(str).replace('NaT', '')
    return df



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject


class CellConfig(BaseObject):
    """Wrapper for Vega-Lite CellConfig definition.
    
    Attributes
    ----------
    clip: Bool
        
    fill: Unicode
        The fill color.
    fillOpacity: CFloat
        The fill opacity (value between [0,1]).
    height: CFloat
        
    stroke: Unicode
        The stroke color.
    strokeDash: List(CFloat)
        An array of alternating stroke, space lengths for creating dashed or dotted lines.
    strokeDashOffset: CFloat
        The offset (in pixels) into which to begin drawing with the stroke dash array.
    strokeOpacity: CFloat
        The stroke opacity (value between [0,1]).
    strokeWidth: CFloat
        The stroke width, in pixels.
    width: CFloat
        
    """
    clip = T.Bool(allow_none=True, default_value=None)
    fill = T.Unicode(allow_none=True, default_value=None, help="""The fill color.""")
    fillOpacity = T.CFloat(allow_none=True, default_value=None, help="""The fill opacity (value between [0,1]).""")
    height = T.CFloat(allow_none=True, default_value=None)
    stroke = T.Unicode(allow_none=True, default_value=None, help="""The stroke color.""")
    strokeDash = T.List(T.CFloat(), allow_none=True, default_value=None, help="""An array of alternating stroke, space lengths for creating dashed or dotted lines.""")
    strokeDashOffset = T.CFloat(allow_none=True, default_value=None, help="""The offset (in pixels) into which to begin drawing with the stroke dash array.""")
    strokeOpacity = T.CFloat(allow_none=True, default_value=None, help="""The stroke opacity (value between [0,1]).""")
    strokeWidth = T.CFloat(allow_none=True, default_value=None, help="""The stroke width, in pixels.""")
    width = T.CFloat(allow_none=True, default_value=None)
    
    def __init__(self, clip=None, fill=None, fillOpacity=None, height=None, stroke=None, strokeDash=None, strokeDashOffset=None, strokeOpacity=None, strokeWidth=None, width=None, **kwargs):
        kwds = dict(clip=clip, fill=fill, fillOpacity=fillOpacity, height=height, stroke=stroke, strokeDash=strokeDash, strokeDashOffset=strokeDashOffset, strokeOpacity=strokeOpacity, strokeWidth=strokeWidth, width=width)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(CellConfig, self).__init__(**kwargs)


"""Code for Example Plots"""
import os
import json

JSON_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), 'json'))


def load_example(filename):
    """Load the JSON dict corresponding to the given filename"""
    if filename not in os.listdir(JSON_DIR):
        raise ValueError("Example='{0}' not valid.".format(filename))

    with open(os.path.join(JSON_DIR, filename)) as f:
        return json.load(f)


def iter_examples():
    """Iterate all example files & their contents

    Iterator returns tuples of (filename, JSON_dict)
    """
    for filename in os.listdir(JSON_DIR):
        yield filename, load_example(filename)



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject
from .aggregateop import AggregateOp
from .sortorder import SortOrder


class SortField(BaseObject):
    """Wrapper for Vega-Lite SortField definition.
    
    Attributes
    ----------
    field: Unicode
        The field name to aggregate over.
    op: AggregateOp
        The sort aggregation operator.
    order: SortOrder
        
    """
    field = T.Unicode(allow_none=True, default_value=None, help="""The field name to aggregate over.""")
    op = AggregateOp(allow_none=True, default_value=None, help="""The sort aggregation operator.""")
    order = SortOrder(allow_none=True, default_value=None)
    
    def __init__(self, field=None, op=None, order=None, **kwargs):
        kwds = dict(field=field, op=op, order=order)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(SortField, self).__init__(**kwargs)


import pytest
import warnings
import json

import numpy as np
import pandas as pd

from .. import *
from .. import schema
from ..utils import parse_shorthand, infer_vegalite_type
from ..datasets import connection_ok
from ..utils._py3k_compat import PY2


def test_chart_url_input():
    url = 'http://vega.github.io/vega-lite/data/'
    chart1 = Chart(Data(url=url))
    chart2 = Chart(url)

    assert chart1.to_dict() == chart2.to_dict()

    assert chart1.to_altair() == chart2.to_altair()


def test_chart_to_html():
    chart = Chart().encode(x='blah:Q')
    html = chart.to_html(title='My Chart')
    assert "<title>My Chart</title>" in html

    html = chart.to_html(template="{title}<@>{spec}")
    title, spec = html.split('<@>')
    assert json.loads(spec) == chart.to_dict()
    assert title == "Vega-Lite Chart"


def test_encode_update():
    # Test that encode updates rather than overwrites
    chart1 = Chart().encode(x='blah:Q').encode(y='blah:Q')
    chart2 = Chart().encode(x='blah:Q', y='blah:Q')

    assert chart1.to_dict() == chart2.to_dict()


def test_configure_update():
    # Test that configure updates rather than overwrites
    chart1 = Chart().configure(MarkConfig(color='red'))\
                    .configure(background='red')
    chart2 = Chart().configure(MarkConfig(color='red'), background='red')

    assert chart1.to_dict() == chart2.to_dict()


def test_configure_axis_update():
    chart1 = Chart().configure_axis(axisColor='red')\
                    .configure_axis(axisWidth=100)
    chart2 = Chart().configure_axis(axisColor='red',
                                    axisWidth=100)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_cell_update():
    chart1 = Chart().configure_cell(stroke='red')\
                    .configure_cell(height=100)
    chart2 = Chart().configure_cell(stroke='red',
                                    height=100)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_legend_update():
    chart1 = Chart().configure_legend(gradientStrokeColor='red')\
                    .configure_legend(gradientHeight=100)
    chart2 = Chart().configure_legend(gradientStrokeColor='red',
                                      gradientHeight=100)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_mark_update():
    chart1 = Chart().configure_mark(color='red')\
                    .configure_mark(angle=90)
    chart2 = Chart().configure_mark(color='red',
                                    angle=90)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_scale_update():
    chart1 = Chart().configure_scale(bandSize=50)\
                    .configure_scale(round=True)
    chart2 = Chart().configure_scale(bandSize=50,
                                     round=True)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_facet_axis_update():
    chart1 = Chart().configure_facet_axis(axisColor='red')\
                    .configure_facet_axis(axisWidth=100)
    chart2 = Chart().configure_facet_axis(axisColor='red',
                                          axisWidth=100)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_facet_cell_update():
    chart1 = Chart().configure_facet_cell(stroke='red')\
                    .configure_facet_cell(height=100)
    chart2 = Chart().configure_facet_cell(stroke='red',
                                          height=100)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_facet_grid_update():
    chart1 = Chart().configure_facet_grid(color='red')\
                    .configure_facet_grid(offset=100)
    chart2 = Chart().configure_facet_grid(color='red',
                                          offset=100)
    assert chart1.to_dict() == chart2.to_dict()


def test_configure_facet_scale_update():
    chart1 = Chart().configure_facet_scale(padding=50)\
                    .configure_facet_scale(round=True)
    chart2 = Chart().configure_facet_scale(padding=50,
                                           round=True)
    assert chart1.to_dict() == chart2.to_dict()


def test_transform_update():
    # Test that transform updates rather than overwrites
    formula = Formula(field='gender', expr='datum.sex == 2 ? "Female":"Male"')
    chart1 = Chart().transform_data(filter='datum.year==2000')\
                    .transform_data(calculate=[formula])

    chart2 = Chart().transform_data(filter='datum.year==2000',
                                    calculate=[formula])

    assert chart1.to_dict() == chart2.to_dict()


def test_load_vegalite_spec():
    df = pd.DataFrame({'x':[1,2,3], 'y':[4,5,6]})
    obj = Chart(df).mark_point().encode(x='x', y='y')
    obj2 = load_vegalite_spec(obj.to_dict())
    assert obj.to_dict() == obj2.to_dict()


def test_to_altair():
    df = pd.DataFrame({'x':[1,2,3], 'y':[4,5,6]})
    obj = Chart(df).mark_point().encode(x='x', y='y')

    code = obj.to_altair(data='df')
    obj2 = eval(code)

    assert obj.to_dict() == obj2.to_dict()


def test_to_altair_with_methods():
    from ..utils._py3k_compat import PY2
    if PY2:
        code_in = "Chart('http://vega.github.io').mark_point(color=u'red',)"
    else:
        code_in = "Chart('http://vega.github.io').mark_point(color='red',)"
    code_out = eval(code_in).to_altair()
    assert code_in == code_out.replace(' ', '').replace('\n','')


@pytest.mark.skipif(not connection_ok(), reason="No Internet Connection")
def test_to_altair_stocks():
    """Test a more complicated spec for conversion to altair"""
    data = load_dataset('stocks')

    chart = Chart(data).mark_line().encode(
        x='date:T',
        y='price:Q'
    ).transform_data(
        filter="datum.symbol==='GOOG'"
    ).configure(
        mark=MarkConfig(color='red')
    )

    code = chart.to_altair(data='data')
    chart2 = eval(code)

    assert chart.to_dict() == chart2.to_dict()


@pytest.mark.parametrize('mark', schema.Mark().values)
def test_mark_config(mark):
    markmethod = lambda chart: getattr(chart, 'mark_' + mark)
    kwds = dict(color='red', opacity=0.5)

    chart1 = Chart(config=Config(mark=MarkConfig(**kwds)))
    # chart1.mark_circle()
    markmethod(chart1)()
    chart1.encode(x='Horsepower:Q', y='Miles_per_gallon:Q')

    chart2 = Chart()
    # chart2.mark_circle(color='red')
    markmethod(chart2)(**kwds)
    chart2.encode(x='Horsepower:Q', y='Miles_per_gallon:Q')

    chart3 = Chart()
    #chart3.mark_circle().configure_mark(**kwargs)
    markmethod(chart3)().configure_mark(**kwds)
    chart3.encode(x='Horsepower:Q', y='Miles_per_gallon:Q')

    assert chart1.to_dict() == chart2.to_dict()
    assert chart2.to_dict() == chart3.to_dict()


_config_method_params = [
    {'name': 'axis',
     'class': AxisConfig,
     'kwds': dict(axisWidth=1.0, grid=False)
    },
    {'name': 'cell',
     'class': CellConfig,
     'kwds': dict(clip=False, fillOpacity=0.5)
    },
    {'name': 'legend',
     'class': LegendConfig,
     'kwds': dict(orient=u'foo', shortTimeLabels=True)
    },
    {'name': 'scale',
     'class': ScaleConfig,
     'kwds': dict(pointSizeRange=[1.0, 2.0], round=False)
    },
]

@pytest.mark.parametrize('params', _config_method_params)
def test_config_methods(params):
    kwds = params['kwds']
    klass = params['class']
    name = params['name']
    configmethod = lambda chart: getattr(chart, 'configure_'+name)

    chart1 = Chart(config=Config( **{name:klass(**kwds)} ))
    chart2 = Chart().configure( **{name:klass(**kwds)} )
    chart3 = configmethod(Chart())(**kwds)

    assert chart1.to_dict() == chart2.to_dict()
    assert chart2.to_dict() == chart3.to_dict()


def test_config_facet_grid():

    kwds = dict(opacity=0.5, color='red')

    chart1 = Chart(
        config=Config(
            facet=FacetConfig(grid=FacetGridConfig(**kwds))
        )
    )
    chart2 = Chart().configure_facet_grid(**kwds)

    assert chart1.to_dict() == chart2.to_dict()


def test_data_finalization():
    data = pd.DataFrame({'x': np.arange(5),
                         'y': pd.date_range('2016-01-01', freq='D', periods=5),
                         'row': list('ABABA')})
    # Facet containing data and a chart
    chart1 = Chart().mark_point().encode(x='x', y='y')
    facet = FacetedChart(data, spec=chart1).set_facet(row='row')
    D = facet.to_dict()
    assert D['facet']['row']['type'] == 'nominal'
    assert D['spec']['encoding']['x']['type'] == 'quantitative'
    assert D['spec']['encoding']['y']['type'] == 'temporal'

    # Facet containing a chart and no data
    chart1 = Chart().mark_point().encode(x='x', y='y')
    facet = FacetedChart(spec=chart1).set_facet(row='row')
    D = facet.to_dict()
    assert 'type' not in D['facet']['row']
    assert 'type' not in D['spec']['encoding']['x']
    assert 'type' not in D['spec']['encoding']['y']

    # Facet containing data and a layer
    chart1 = Chart().mark_point().encode(x='x', y='y')
    chart2 = Chart().mark_rule().encode(x='average(x)', y='average(y)')
    layer = LayeredChart(layers=[chart1, chart2])
    facet = FacetedChart(data, spec=layer).set_facet(row='row')
    D = facet.to_dict()
    assert D['facet']['row']['type'] == 'nominal'
    for layer in D['spec']['layers']:
        assert layer['encoding']['x']['type'] == 'quantitative'
        assert layer['encoding']['y']['type'] == 'temporal'

    # Facet containing a layer and no data
    chart1 = Chart().mark_point().encode(x='x', y='y')
    chart2 = Chart().mark_rule().encode(x='average(x)', y='average(y)')
    layer = LayeredChart(layers=[chart1, chart2])
    facet = FacetedChart(spec=layer).set_facet(row='row')
    D = facet.to_dict()
    assert 'type' not in D['facet']['row']
    for layer in D['spec']['layers']:
        assert 'type' not in layer['encoding']['x']
        assert 'type' not in layer['encoding']['y']


SAMPLE_CODE = """
Chart(cars).mark_tick().encode(
    x='Miles_per_Gallon',
    y='Origin',
)
"""

@pytest.fixture
def sample_code():
    # In Py2, output strings will be unicode
    if PY2:
        return SAMPLE_CODE.replace("='", "=u'").strip()
    else:
        return SAMPLE_CODE.strip()


@pytest.mark.skipif(not connection_ok(), reason="No Internet Connection")
def test_finalize(sample_code):
    cars = load_dataset('cars')

    # Test that finalize is not called for ``to_altair()`` method
    obj = eval(sample_code)
    assert obj.to_altair(data='cars') == sample_code

    # Confirm that _finalize() changes the state
    assert obj.encoding.x.type is None
    obj._finalize()
    assert obj.encoding.x.type is not None

    # Confirm that finalized object contains correct type information
    D = obj.to_dict(data=False)
    assert D['encoding']['x']['type'] == 'quantitative'
    assert D['encoding']['y']['type'] == 'nominal'


def test_layered_chart_iadd():
    data = pd.DataFrame({'x':np.random.rand(10), 'y':np.random.rand(10)})
    l1 = Chart().mark_line().encode(x='x', y='y')
    l2 = Chart().mark_point().encode(x='x', y='y')

    chart = LayeredChart(data)
    chart += l1
    chart += l2
    chart2 = LayeredChart(data)
    chart2.set_layers(l1, l2)
    assert chart.to_dict()==chart2.to_dict()


def test_chart_add():
    data = pd.DataFrame({'x':np.random.rand(10), 'y':np.random.rand(10)})
    l1 = Chart(data).mark_line().encode(x='x', y='y')
    l2 = Chart(data).mark_point().encode(x='x', y='y')
    
    chart = l1+l2
    chart2 = LayeredChart()
    chart2.set_layers(l1, l2)
    assert chart.to_dict()==chart2.to_dict()



"""Auto-generated Python wrappers for Vega-Lite Schema"""


__all__ = [
             "AggregateOp",
             "Axis",
             "AxisConfig",
             "AxisOrient",
             "Bin",
             "CellConfig",
             "ChannelDefWithLegend",
             "Config",
             "Data",
             "DataFormat",
             "Encoding",
             "ExtendedUnitSpec",
             "Facet",
             "FacetConfig",
             "FacetGridConfig",
             "FacetScaleConfig",
             "FacetSpec",
             "FieldDef",
             "FontStyle",
             "FontWeight",
             "Formula",
             "HorizontalAlign",
             "Interpolate",
             "LayerSpec",
             "Legend",
             "LegendConfig",
             "Mark",
             "MarkConfig",
             "NiceTime",
             "OrderChannelDef",
             "PositionChannelDef",
             "Scale",
             "ScaleConfig",
             "ScaleType",
             "Shape",
             "SortField",
             "SortOrder",
             "StackOffset",
             "TimeUnit",
             "Transform",
             "Type",
             "UnitEncoding",
             "UnitSpec",
             "VerticalAlign",
          ]


from .aggregateop import AggregateOp
from .axis import Axis
from .axisconfig import AxisConfig
from .axisorient import AxisOrient
from .bin import Bin
from .cellconfig import CellConfig
from .channeldefwithlegend import ChannelDefWithLegend
from .config import Config
from .data import Data
from .dataformat import DataFormat
from .encoding import Encoding
from .extendedunitspec import ExtendedUnitSpec
from .facet import Facet
from .facetconfig import FacetConfig
from .facetgridconfig import FacetGridConfig
from .facetscaleconfig import FacetScaleConfig
from .facetspec import FacetSpec
from .fielddef import FieldDef
from .fontstyle import FontStyle
from .fontweight import FontWeight
from .formula import Formula
from .horizontalalign import HorizontalAlign
from .interpolate import Interpolate
from .layerspec import LayerSpec
from .legend import Legend
from .legendconfig import LegendConfig
from .mark import Mark
from .markconfig import MarkConfig
from .nicetime import NiceTime
from .orderchanneldef import OrderChannelDef
from .positionchanneldef import PositionChannelDef
from .scale import Scale
from .scaleconfig import ScaleConfig
from .scaletype import ScaleType
from .shape import Shape
from .sortfield import SortField
from .sortorder import SortOrder
from .stackoffset import StackOffset
from .timeunit import TimeUnit
from .transform import Transform
from .type import Type
from .unitencoding import UnitEncoding
from .unitspec import UnitSpec
from .verticalalign import VerticalAlign



# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T
from ..baseobject import BaseObject


class Formula(BaseObject):
    """Wrapper for Vega-Lite Formula definition.
    
    Attributes
    ----------
    expr: Unicode
        A string containing an expression for the formula.
    field: Unicode
        The field in which to store the computed formula value.
    """
    expr = T.Unicode(allow_none=True, default_value=None, help="""A string containing an expression for the formula.""")
    field = T.Unicode(allow_none=True, default_value=None, help="""The field in which to store the computed formula value.""")
    
    def __init__(self, expr=None, field=None, **kwargs):
        kwds = dict(expr=expr, field=field)
        kwargs.update({k:v for k, v in kwds.items() if v is not None})
        super(Formula, self).__init__(**kwargs)


# This file auto-generated by `generate_schema_interface.py`.
# Do not modify this file directly.

import traitlets as T


class ScaleType(T.Enum):
    """One of ['linear', 'log', 'pow', 'sqrt', 'quantile', 'quantize', 'ordinal', 'time', 'utc']"""
    def __init__(self, default_value=T.Undefined, **metadata):
        super(ScaleType, self).__init__(['linear', 'log', 'pow', 'sqrt', 'quantile', 'quantize', 'ordinal', 'time', 'utc'],
                                    default_value=default_value,
                                    **metadata)